@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2009 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -588,8 +588,10 @@ libbalsa_address_book_ldif_save_address(FILE * stream,
                                         LibBalsaAddress * address)
 {
     if (fseek(stream, -2, SEEK_END) == 0
-        && (fgetc(stream) != '\n' || fgetc(stream) != '\n'))
+        && (fgetc(stream) != '\n' || fgetc(stream) != '\n')) {
+        fseek(stream, 0, SEEK_END);
         fputc('\n', stream);
+    }
 
     lbab_ldif_write_dn(stream, address);
     lbab_ldif_write_givenname(stream, address);
