@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,10 +23,11 @@
 #ifndef _BALSA_FILES_H
 #define _BALSA_FILES_H
 
-#include "config.h"
 #include <glib.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
+#include "body.h"
+#include "libbalsa-vfs.h"
 
 /* filename is the filename (naw!)
  * splice is what to put in between the prefix and the filename, if desired
@@ -43,11 +44,10 @@ gchar *balsa_file_finder(const gchar * filename, const gchar * splice,
 #define balsa_pixmap_finder_no_warn(filename) \
     (balsa_file_finder((filename), "pixmaps", NULL, FALSE))
 
-GdkPixbuf *libbalsa_icon_finder(const char *mime_type, const char *filename,
-				gchar** used_type, GtkIconSize size);
-
-void libbalsa_fill_vfs_menu_by_content_type(GtkMenu * menu,
-					    const gchar * content_type,
-					    GCallback callback, gpointer data);
+GdkPixbuf *libbalsa_icon_finder(GtkWidget         * widget,
+                                const char        * mime_type,
+                                const LibbalsaVfs * for_file,
+                                gchar            ** used_type,
+                                GtkIconSize         size);
 
 #endif
