@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2009 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -395,7 +395,7 @@ append_header_names(LibBalsaCondition *cond, GString *res)
     for (i=0; i<ELEMENTS(header_name_map); ++i) {
         if (CONDITION_CHKMATCH(cond, header_name_map[i].header)) {
             if (res->len>0) {
-                g_string_append_printf(res, _(",%s"),
+                g_string_append_printf(res, ",%s",
                                        _(header_name_map[i].header_name));
             } else {
                 res = g_string_append(res, _(header_name_map[i].header_name));
@@ -423,7 +423,7 @@ append_flag_names(LibBalsaCondition *cond, GString *res)
             if (res->len == len) {
                 res = g_string_append(res, _(flag_name_map[i].flag_name));
             } else {
-                g_string_printf(res, _(",%s"),
+                g_string_printf(res, ",%s",
                                 _(flag_name_map[i].flag_name));
             }
         }
@@ -975,10 +975,17 @@ gboolean
 libbalsa_filter_export_sieve(LibBalsaFilter* fil, gchar* filename)
 {
     FILE * fp;
+#if 0
+    GString * buffer;
+#endif
     gint nb = 0;
 
     fp=fopen(filename,"w");
     if (!fp) return FALSE;
+#if 0
+    buffer=g_string_new("# Sieve script automatically generated by Balsa "
+                        "(Exporting a Balsa filter)\n");
+#endif
     g_warning("%s: FIXME!\n", __func__);
 #if 0
     
@@ -1023,8 +1030,8 @@ libbalsa_filter_export_sieve(LibBalsaFilter* fil, gchar* filename)
     }
     nb=fwrite(buffer->str,buffer->len,1,fp);
     g_string_free(buffer,TRUE);
-    if (fclose(fp)!=0) nb=0;
 #endif
+    if (fclose(fp)!=0) nb=0;
     return nb==1 ? TRUE : FALSE;
 }             /* end of filter_export_sieve */
 
