@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2009 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,13 +23,13 @@
 /*
  * Support for HTML mail parts.
  *
- * Balsa supports three HTML engines: GtkHtml-2, GtkHTML-3, and WebKit.
+ * Balsa supports three HTML engines: GtkHtml-2, GtkHTML-4, and WebKit.
  * The symbol HAVE_HTML_WIDGET is defined if HTML support is requested at
  * configure time, and the requested engine is available.
  *
  * This file contains all code that depends on which widget is being
  * used. Elsewhere, HTML support code should be conditional on
- * HAVE_HTML_WIDGET, but none of HAVE_GTKHTML2, HAVE_GTKHTML3, or
+ * HAVE_HTML_WIDGET, but none of HAVE_GTKHTML2, HAVE_GTKHTML4, or
  * HAVE_WEBKIT should be referenced outside this file.
  *
  * As of this writing (2010-01), WebKit offers the most complete API,
@@ -79,11 +79,794 @@ lbh_get_body_content(LibBalsaMessageBody * body, gchar ** buf)
 }
 
 # if defined(HAVE_WEBKIT)
+#  if defined(USE_WEBKIT2)
 
 /*
- * Experimental support for WebKit.
+ * Experimental support for WebKit2.
  */
 
+#define DEBUG_WEBKIT2 FALSE
+#if DEBUG_WEBKIT2
+#define d(x) x
+#else
+#define d(x)
+#endif
+
+#include <webkit2/webkit2.h>
+#include <JavaScriptCore/JavaScript.h>
+
+typedef struct {
+    LibBalsaMessageBody  *body;
+    LibBalsaHtmlCallback  hover_cb;
+    LibBalsaHtmlCallback  clicked_cb;
+    GtkWidget            *info_bar;
+    WebKitWebView        *web_view;
+    gchar                *uri;
+    LibBalsaHtmlSearchCallback search_cb;
+    gpointer                   search_cb_data;
+    gchar                    * search_text;
+} LibBalsaWebKitInfo;
+
+#define LIBBALSA_HTML_INFO "libbalsa-webkit2-info"
+
+/*
+ * Unlike older HTML widgets, webkit2 wants UTF-8 text
+ */
+static gssize
+lbh_get_body_content_utf8(LibBalsaMessageBody  * body,
+                          gchar               ** utf8_text)
+{
+    gchar *text;
+    gssize len;
+    const gchar *charset;
+    gsize utf8_len;
+
+    len = lbh_get_body_content(body, &text);
+    if (len < 0)
+        return len;
+
+    charset = libbalsa_message_body_charset(body);
+    if (charset) {
+        *utf8_text = g_convert(text, len, "UTF-8", charset,
+                               NULL, &utf8_len, NULL);
+        if (*utf8_text) {
+            /* Success! */
+            g_free(text);
+            return utf8_len;
+        }
+    }
+
+    /* No charset, or g_convert failed; just make sure it's UTF-8 */
+    libbalsa_utf8_sanitize(&text, TRUE, NULL);
+    *utf8_text = text;
+
+    return strlen(text);
+}
+
+/*
+ * GDestroyNotify func
+ */
+static void
+lbh_webkit_info_free(LibBalsaWebKitInfo * info)
+{
+    if (info->uri) {
+        g_free(info->uri);
+        (*info->hover_cb) (NULL);
+    }
+
+    g_free(info->search_text);
+    g_free(info);
+}
+
+/*
+ * Callback for the "mouse-target-changed" signal
+ */
+static void
+lbh_mouse_target_changed_cb(WebKitWebView       * web_view,
+                            WebKitHitTestResult * hit_test_result,
+                            guint                 modifiers,
+                            gpointer              data)
+{
+    LibBalsaWebKitInfo *info = data;
+    const gchar *uri;
+
+    uri = webkit_hit_test_result_get_link_uri(hit_test_result);
+
+    if ((!uri && !info->uri)
+        || (uri && info->uri && !strcmp(uri, info->uri)))
+        /* No change */
+        return;
+
+    if (info->uri) {
+        g_free(info->uri);
+        info->uri = NULL;
+        (*info->hover_cb) (NULL);
+    }
+
+    if (uri) {
+        info->uri = g_strdup(uri);
+        (*info->hover_cb) (uri);
+    }
+}
+
+/*
+ * Callback for the "decide-policy" signal
+ *
+ * First, handlers for the three types of decision
+ */
+
+static void
+lbh_navigation_policy_decision(WebKitPolicyDecision * decision,
+                               gpointer               data)
+{
+    WebKitNavigationPolicyDecision *navigation_decision =
+        WEBKIT_NAVIGATION_POLICY_DECISION(decision);
+    LibBalsaWebKitInfo *info = data;
+    WebKitURIRequest *request;
+
+    switch (webkit_navigation_policy_decision_get_navigation_type
+            (navigation_decision)) {
+    case WEBKIT_NAVIGATION_TYPE_OTHER:
+    case WEBKIT_NAVIGATION_TYPE_RELOAD:
+        d(g_print("%s type %d, used\n", __func__,
+                  webkit_navigation_policy_decision_get_navigation_type
+                  (navigation_decision)));
+        webkit_policy_decision_use(decision);
+        break;
+    case WEBKIT_NAVIGATION_TYPE_LINK_CLICKED:
+        request =
+            webkit_navigation_policy_decision_get_request
+            (navigation_decision);
+        d(g_print("%s clicked %s\n", __func__,
+                  webkit_uri_request_get_uri(request)));
+        (*info->clicked_cb) (webkit_uri_request_get_uri(request));
+    default:
+        d(g_print("%s type %d, ignored\n", __func__,
+                  webkit_navigation_policy_decision_get_navigation_type
+                  (navigation_decision)));
+        webkit_policy_decision_ignore(decision);
+    }
+}
+
+static void
+lbh_new_window_policy_decision(WebKitPolicyDecision * decision,
+                               gpointer               data)
+{
+    WebKitNavigationPolicyDecision *navigation_decision =
+        WEBKIT_NAVIGATION_POLICY_DECISION(decision);
+    LibBalsaWebKitInfo *info = data;
+    WebKitURIRequest *request;
+
+    switch (webkit_navigation_policy_decision_get_navigation_type
+            (navigation_decision)) {
+    case WEBKIT_NAVIGATION_TYPE_LINK_CLICKED:
+        request =
+            webkit_navigation_policy_decision_get_request
+            (navigation_decision);
+        d(g_print("%s clicked %s\n", __func__,
+                  webkit_uri_request_get_uri(request)));
+        (*info->clicked_cb) (webkit_uri_request_get_uri(request));
+    default:
+        d(g_print("%s type %d, ignored\n", __func__,
+                  webkit_navigation_policy_decision_get_navigation_type
+                  (navigation_decision)));
+
+        webkit_policy_decision_ignore(decision);
+    }
+}
+
+static void
+lbh_response_policy_decision(WebKitPolicyDecision * decision,
+                             gpointer               data)
+{
+    d(g_print("%s uri %s, ignored\n", __func__,
+              webkit_uri_request_get_uri
+              (webkit_response_policy_decision_get_request
+               (WEBKIT_RESPONSE_POLICY_DECISION(decision)))));
+    webkit_policy_decision_ignore(decision);
+}
+
+static gboolean
+lbh_decide_policy_cb(WebKitWebView           * web_view,
+                     WebKitPolicyDecision    * decision,
+                     WebKitPolicyDecisionType  decision_type,
+                     gpointer                  data)
+{
+    switch (decision_type) {
+    case WEBKIT_POLICY_DECISION_TYPE_NAVIGATION_ACTION:
+        lbh_navigation_policy_decision(decision, data);
+        break;
+    case WEBKIT_POLICY_DECISION_TYPE_NEW_WINDOW_ACTION:
+        lbh_new_window_policy_decision(decision, data);
+        break;
+    case WEBKIT_POLICY_DECISION_TYPE_RESPONSE:
+        lbh_response_policy_decision(decision, data);
+        break;
+    default:
+        /* Making no decision results in
+         * webkit_policy_decision_use(). */
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
+/*
+ * Show the GtkInfoBar for asking about downloading images
+ *
+ * First two signal callbacks
+ */
+
+static void
+lbh_info_bar_response_cb(GtkInfoBar * info_bar,
+                         gint response_id, gpointer data)
+{
+    LibBalsaWebKitInfo *info = data;
+
+    if (response_id == GTK_RESPONSE_OK) {
+        gchar *text;
+
+        if (lbh_get_body_content_utf8(info->body, &text) >= 0) {
+            WebKitSettings *settings;
+
+            settings = webkit_web_view_get_settings(info->web_view);
+            webkit_settings_set_auto_load_images(settings, TRUE);
+            webkit_web_view_load_html(info->web_view, text, NULL);
+            g_free(text);
+        }
+    }
+
+    gtk_widget_destroy(info->info_bar);
+    info->info_bar = NULL;
+}
+
+static void
+lbh_info_bar_realize_cb(GtkInfoBar * info_bar)
+{
+    gtk_info_bar_set_default_response(info_bar, GTK_RESPONSE_CLOSE);
+}
+
+static GtkWidget *
+lbh_info_bar(LibBalsaWebKitInfo * info)
+{
+    GtkWidget *info_bar_widget;
+    GtkInfoBar *info_bar;
+    GtkWidget *label;
+    GtkWidget *content_area;
+    static const gchar text[] =
+                 N_("This message part contains images "
+                    "from a remote server. "
+                    "To protect your privacy, "
+                    "Balsa has not downloaded them. "
+                    "You may choose to download them "
+                    "if you trust the server.");
+
+    info_bar_widget =
+        gtk_info_bar_new_with_buttons(_("_Download images"),
+                                     GTK_RESPONSE_OK,
+                                     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+                                     NULL);
+
+    info_bar = GTK_INFO_BAR(info_bar_widget);
+
+    label = gtk_label_new(_(text));
+    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+
+    content_area = gtk_info_bar_get_content_area(info_bar);
+    gtk_container_add(GTK_CONTAINER(content_area), label);
+
+    g_signal_connect(info_bar, "realize",
+                     G_CALLBACK(lbh_info_bar_realize_cb), info);
+    g_signal_connect(info_bar, "response",
+                     G_CALLBACK(lbh_info_bar_response_cb), info);
+    gtk_info_bar_set_message_type(info_bar, GTK_MESSAGE_QUESTION);
+
+    return info_bar_widget;
+}
+
+/*
+ * Callback for the "resource-load-started" signal
+ */
+static void
+lbh_resource_load_started_cb(WebKitWebView     * web_view,
+                             WebKitWebResource * resource,
+                             WebKitURIRequest  * request,
+                             gpointer            data)
+{
+    LibBalsaWebKitInfo *info = data;
+    const gchar *uri;
+
+    uri = webkit_uri_request_get_uri(request);
+    if (!g_ascii_strcasecmp(uri, "about:blank"))
+        return;
+
+    if (info->info_bar) {
+        /* web_view is loading an image from its cache, so we do not
+         * need to ask the user for permission to download */
+        gtk_widget_destroy(info->info_bar);
+        info->info_bar = NULL;
+    }
+}
+
+/*
+ * Callback for the "web-process-crashed" signal
+ */
+static gboolean
+lbh_web_process_crashed_cb(WebKitWebView * web_view,
+                           gpointer        data)
+{
+    d(g_print("%s\n", __func__));
+    return FALSE;
+}
+
+/*
+ * WebKitURISchemeRequestCallback for "cid:" URIs
+ */
+static void
+lbh_cid_cb(WebKitURISchemeRequest * request,
+           gpointer                 data)
+{
+    LibBalsaWebKitInfo *info = *(LibBalsaWebKitInfo **) data;
+    const gchar *path;
+    LibBalsaMessageBody *body;
+
+    path = webkit_uri_scheme_request_get_path(request);
+    d(g_print("%s path %s\n", __func__, path));
+
+    if ((body =
+         libbalsa_message_get_part_by_id(info->body->message, path))) {
+        gchar *content;
+        gssize len;
+
+        len = libbalsa_message_body_get_content(body, &content, NULL);
+        if (len > 0) {
+            GInputStream *stream;
+            gchar *mime_type;
+
+            stream =
+                g_memory_input_stream_new_from_data(content, len, g_free);
+            mime_type = libbalsa_message_body_get_mime_type(body);
+            webkit_uri_scheme_request_finish(request, stream, len,
+                                             mime_type);
+            g_object_unref(stream);
+            g_free(mime_type);
+        }
+    }
+}
+
+/*
+ * Callback for the "context-menu" signal
+ */
+static gboolean
+lbh_context_menu_cb(WebKitWebView       * web_view,
+                    WebKitContextMenu   * context_menu,
+                    GdkEvent            * event,
+                    WebKitHitTestResult * hit_test_result,
+                    gpointer              data)
+{
+    GtkWidget *parent;
+    gboolean retval;
+
+    parent = gtk_widget_get_parent(GTK_WIDGET(web_view));
+    /* The signal is asynchronous, so gtk_get_current_event() gets NULL;
+     * we pass the event to the popup-menu handler: */
+    g_object_set_data(G_OBJECT(parent), LIBBALSA_HTML_POPUP_EVENT, event);
+
+    g_signal_emit_by_name(parent, "popup-menu", &retval);
+
+    g_object_set_data(G_OBJECT(parent), LIBBALSA_HTML_POPUP_EVENT, NULL);
+
+    return retval;
+}
+
+/* Create a new WebKitWebView widget:
+ * body 		LibBalsaMessageBody that belongs to the
+ *                      LibBalsaMessage from which to extract any
+ *			HTML objects (by url);
+ * hover_cb             callback for link-hover signal;
+ * clicked_cb	        callback for the "link-clicked" signal;
+ */
+
+GtkWidget *
+libbalsa_html_new(LibBalsaMessageBody * body,
+                  LibBalsaHtmlCallback  hover_cb,
+                  LibBalsaHtmlCallback  clicked_cb)
+{
+    gchar *text;
+    gssize len;
+    GtkWidget *widget;
+    GtkWidget *vbox;
+    WebKitWebView *web_view;
+    static LibBalsaWebKitInfo *info;
+    static gboolean have_registered_cid = FALSE;
+    WebKitSettings *settings;
+    static const gchar cid_regex[] =
+        "<[^>]*src\\s*=\\s*['\"]?\\s*cid:";
+    static const gchar src_regex[] =
+        "<[^>]*src\\s*=\\s*['\"]?\\s*[^c][^i][^d][^:]";
+
+    len = lbh_get_body_content_utf8(body, &text);
+    if (len < 0)
+        return NULL;
+
+    info = g_new(LibBalsaWebKitInfo, 1);
+    info->body            = body;
+    info->hover_cb        = hover_cb;
+    info->clicked_cb      = clicked_cb;
+    info->info_bar        = NULL;
+    info->uri             = NULL;
+    info->search_text     = NULL;
+
+    widget = webkit_web_view_new();
+    /* WebkitWebView is uncontrollably scrollable, so if we don't set a
+     * minimum size it may be just a few pixels high. */
+    gtk_widget_set_size_request(widget, -1, 200);
+
+    info->web_view = web_view = WEBKIT_WEB_VIEW(widget);
+    g_object_set_data_full(G_OBJECT(web_view), LIBBALSA_HTML_INFO, info,
+                           (GDestroyNotify) lbh_webkit_info_free);
+
+    if (!have_registered_cid) {
+        WebKitWebContext *context;
+        /* Apparently, WebKitWebContext is static, and does not like to
+         * have the scheme registered many times (13? 15?), after which
+         * the web process crashes and does not get respawned.
+         * We register it once with the address of a static pointer to
+         * LibBalsaWebKitInfo. */
+
+        context = webkit_web_view_get_context(web_view);
+        webkit_web_context_register_uri_scheme(context, "cid", lbh_cid_cb,
+                                               &info, NULL);
+        have_registered_cid = TRUE;
+    }
+
+    settings = webkit_web_view_get_settings(web_view);
+    webkit_settings_set_enable_plugins(settings, FALSE);
+    webkit_settings_set_auto_load_images
+        (settings,
+         g_regex_match_simple(cid_regex, text, G_REGEX_CASELESS, 0));
+
+    g_signal_connect(web_view, "mouse-target-changed",
+                     G_CALLBACK(lbh_mouse_target_changed_cb), info);
+    g_signal_connect(web_view, "decide-policy",
+                     G_CALLBACK(lbh_decide_policy_cb), info);
+    g_signal_connect(web_view, "resource-load-started",
+                     G_CALLBACK(lbh_resource_load_started_cb), info);
+    g_signal_connect(web_view, "web-process-crashed",
+                     G_CALLBACK(lbh_web_process_crashed_cb), info);
+    g_signal_connect(web_view, "context-menu",
+                     G_CALLBACK(lbh_context_menu_cb), info);
+
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+    g_object_set_data(G_OBJECT(vbox), "libbalsa-html-web-view", web_view);
+    gtk_box_pack_end(GTK_BOX(vbox), widget, TRUE, TRUE, 0);
+
+    /* Simple check for possible resource requests: */
+    if (g_regex_match_simple(src_regex, text, G_REGEX_CASELESS, 0)) {
+        info->info_bar = lbh_info_bar(info);
+        gtk_box_pack_start(GTK_BOX(vbox), info->info_bar, FALSE, FALSE, 0);
+    }
+
+    webkit_web_view_load_html(web_view, text, NULL);
+    g_free(text);
+
+    return vbox;
+}
+
+void
+libbalsa_html_to_string(gchar ** text, size_t len)
+{
+    return; /* this widget does not support conversion to a string. The
+             * string won't be altered. Other alternative would be to set
+             * it to an empty string. */
+}
+
+/*
+ * We may be passed either the WebKitWebView or its container:
+ */
+static gboolean
+lbh_get_web_view(GtkWidget * widget, WebKitWebView ** web_view)
+{
+    if (!WEBKIT_IS_WEB_VIEW(widget))
+        widget =
+            g_object_get_data(G_OBJECT(widget), "libbalsa-html-web-view");
+
+    *web_view = (WebKitWebView *) widget;
+
+    return WEBKIT_IS_WEB_VIEW(*web_view);
+}
+
+/*
+ * Does the widget support zoom?
+ */
+gboolean
+libbalsa_html_can_zoom(GtkWidget * widget)
+{
+    WebKitWebView *web_view;
+
+    return lbh_get_web_view(widget, &web_view);
+}
+
+/*
+ * Zoom the widget.
+ */
+#define LIBBALSA_HTML_ZOOM_FACTOR 1.2
+void
+libbalsa_html_zoom(GtkWidget * widget, gint in_out)
+{
+    WebKitWebView *web_view;
+
+    if (lbh_get_web_view(widget, &web_view)) {
+        gdouble zoom_level;
+
+        zoom_level = webkit_web_view_get_zoom_level(web_view);
+
+        switch (in_out) {
+        case +1:
+            zoom_level *= LIBBALSA_HTML_ZOOM_FACTOR;
+            break;
+        case -1:
+            zoom_level /= LIBBALSA_HTML_ZOOM_FACTOR;
+            break;
+        case 0:
+            zoom_level = 1.0;
+            break;
+        default:
+            break;
+        }
+
+        webkit_web_view_set_zoom_level(web_view, zoom_level);
+    }
+}
+
+/*
+ * Does the widget support selecting text?
+ */
+gboolean
+libbalsa_html_can_select(GtkWidget * widget)
+{
+    WebKitWebView *web_view;
+
+    return lbh_get_web_view(widget, &web_view);
+}
+
+/*
+ * Select all the text.
+ */
+void
+libbalsa_html_select_all(GtkWidget * widget)
+{
+    WebKitWebView *web_view;
+
+    if (lbh_get_web_view(widget, &web_view))
+        webkit_web_view_execute_editing_command
+            (web_view, WEBKIT_EDITING_COMMAND_SELECT_ALL);
+}
+
+/*
+ * Copy selected text to the clipboard.
+ */
+void
+libbalsa_html_copy(GtkWidget * widget)
+{
+    WebKitWebView *web_view;
+
+    if (lbh_get_web_view(widget, &web_view))
+        webkit_web_view_execute_editing_command
+            (web_view, WEBKIT_EDITING_COMMAND_COPY);
+}
+
+/*
+ * Does the widget support searching text?
+ */
+gboolean
+libbalsa_html_can_search(GtkWidget * widget)
+{
+    WebKitWebView *web_view;
+
+    return lbh_get_web_view(widget, &web_view);
+}
+
+/*
+ * Search for the text; if text is empty, call the callback with TRUE
+ * (for consistency with GtkTextIter methods).
+ */
+
+static void
+lbh_search_failed_to_find_text_cb(WebKitFindController * controller,
+                                  gpointer               data)
+{
+    LibBalsaWebKitInfo *info = data;
+
+    (*info->search_cb)(info->search_text, FALSE, info->search_cb_data);
+}
+
+static void
+lbh_search_found_text_cb(WebKitFindController *controller,
+                         guint                 match_count,
+                         gpointer              data)
+{
+    LibBalsaWebKitInfo *info = data;
+
+    (*info->search_cb)(info->search_text, TRUE, info->search_cb_data);
+}
+
+static void
+lbh_search_init(LibBalsaWebKitInfo       * info,
+                WebKitFindController     * controller,
+                const gchar              * text,
+                gboolean                   find_forward,
+                gboolean                   wrap,
+                LibBalsaHtmlSearchCallback search_cb,
+                gpointer                   cb_data)
+{
+    guint32 find_options;
+
+    if (!info->search_text) {
+        /* First search */
+        g_signal_connect(controller, "failed-to-find-text",
+                         G_CALLBACK(lbh_search_failed_to_find_text_cb),
+                         info);
+        g_signal_connect(controller, "found-text",
+                         G_CALLBACK(lbh_search_found_text_cb),
+                         info);
+    }
+
+    g_free(info->search_text);
+    info->search_text = g_strdup(text);
+    info->search_cb = search_cb;
+    info->search_cb_data = cb_data;
+
+    if (!*text) {
+        webkit_find_controller_search_finish(controller);
+        (*search_cb)(text, TRUE, cb_data);
+        return;
+    }
+
+    find_options = WEBKIT_FIND_OPTIONS_CASE_INSENSITIVE;
+    if (!find_forward)
+        find_options |= WEBKIT_FIND_OPTIONS_BACKWARDS;
+    if (wrap)
+        find_options |= WEBKIT_FIND_OPTIONS_WRAP_AROUND;
+
+    webkit_find_controller_search(controller, text, find_options,
+                                  G_MAXUINT);
+}
+
+static void
+lbh_search_continue(WebKitFindController * controller,
+                    const gchar          * text,
+                    gboolean               find_forward,
+                    gboolean               wrap)
+{
+    guint32 find_options;
+    guint32 orig_find_options;
+
+    orig_find_options = find_options =
+        webkit_find_controller_get_options(controller);
+
+    if (!find_forward)
+        find_options |= WEBKIT_FIND_OPTIONS_BACKWARDS;
+    else
+        find_options &= ~WEBKIT_FIND_OPTIONS_BACKWARDS;
+
+    if (wrap)
+        find_options |= WEBKIT_FIND_OPTIONS_WRAP_AROUND;
+    else
+        find_options &= ~WEBKIT_FIND_OPTIONS_WRAP_AROUND;
+
+    if (find_options != orig_find_options) {
+        /* No setter for find-options, so we start a new search */
+        webkit_find_controller_search(controller, text, find_options,
+                                      G_MAXUINT);
+    } else {
+        /* OK to use next/previous methods */
+        if (find_forward)
+            webkit_find_controller_search_next(controller);
+        else
+            webkit_find_controller_search_previous(controller);
+    }
+}
+
+void
+libbalsa_html_search(GtkWidget                * widget,
+                     const gchar              * text,
+                     gboolean                   find_forward,
+                     gboolean                   wrap,
+                     LibBalsaHtmlSearchCallback search_cb,
+                     gpointer                   cb_data)
+{
+    WebKitWebView *web_view;
+    WebKitFindController *controller;
+    LibBalsaWebKitInfo *info;
+
+    if (!lbh_get_web_view(widget, &web_view)) {
+        return;
+    }
+
+    info = g_object_get_data(G_OBJECT(web_view), LIBBALSA_HTML_INFO);
+    controller = webkit_web_view_get_find_controller(web_view);
+
+    if (!info->search_text || strcmp(text, info->search_text)) {
+        lbh_search_init(info, controller, text, find_forward, wrap,
+                        search_cb, cb_data);
+    } else {
+        lbh_search_continue(controller, text, find_forward, wrap);
+    }
+}
+
+/*
+ * We do not need selection bounds.
+ */
+gboolean
+libbalsa_html_get_selection_bounds(GtkWidget    * widget,
+                                   GdkRectangle * selection_bounds)
+{
+    return FALSE;
+}
+
+/*
+ * Get the WebKitWebView widget from the container; we need to connect
+ * to its "populate-popup" signal.
+ */
+GtkWidget *
+libbalsa_html_popup_menu_widget(GtkWidget * widget)
+{
+    return NULL;
+}
+
+GtkWidget *
+libbalsa_html_get_view_widget(GtkWidget * widget)
+{
+    WebKitWebView *web_view;
+
+    return lbh_get_web_view(widget, &web_view) ?
+        GTK_WIDGET(web_view) : NULL;
+}
+
+/*
+ * Does the widget support printing?
+ */
+gboolean
+libbalsa_html_can_print(GtkWidget * widget)
+{
+    WebKitWebView *web_view;
+
+    return lbh_get_web_view(widget, &web_view);
+}
+
+/*
+ * Print the widget's content.
+ */
+void
+libbalsa_html_print(GtkWidget * widget)
+{
+    WebKitWebView *web_view;
+
+    if (lbh_get_web_view(widget, &web_view)) {
+        static GtkPrintSettings *settings = NULL;
+        WebKitPrintOperation *print_operation;
+        WebKitPrintOperationResponse response;
+
+        if (!settings)
+            settings = gtk_print_settings_new();
+
+        print_operation = webkit_print_operation_new(web_view);
+        webkit_print_operation_set_print_settings(print_operation,
+                                                  settings);
+        response =
+            webkit_print_operation_run_dialog(print_operation, NULL);
+        if (response != WEBKIT_PRINT_OPERATION_RESPONSE_CANCEL) {
+            g_object_unref(settings);
+            settings =
+                webkit_print_operation_get_print_settings(print_operation);
+            g_object_ref(settings);
+        }
+        g_object_unref(print_operation);
+    }
+}
+
+#  else                         /* defined(USE_WEBKIT2) */
 #include <webkit/webkit.h>
 #include <JavaScriptCore/JavaScript.h>
 
@@ -92,10 +875,9 @@ typedef struct {
     LibBalsaHtmlCallback  hover_cb;
     LibBalsaHtmlCallback  clicked_cb;
     WebKitWebFrame       *frame;
-#if GTK_CHECK_VERSION(2, 18, 0)
     gboolean              download_images;
-    GtkWidget            *info_bar_widget;
-#endif /* GTK_CHECK_VERSION(2, 18, 0) */
+    GtkWidget            *vbox;
+    gboolean              has_info_bar;
     WebKitWebView        *web_view;
 } LibBalsaWebKitInfo;
 
@@ -159,12 +941,81 @@ lbh_navigation_policy_decision_requested_cb(WebKitWebView             * web_view
     return FALSE;
 }
 
+/*
+ * Show the GtkInfoBar for asking about downloading images
+ */
+
+static void
+lbh_info_bar_response_cb(GtkInfoBar * info_bar,
+                         gint response_id, gpointer user_data)
+{
+    LibBalsaWebKitInfo *info = user_data;
+
+    if (response_id == GTK_RESPONSE_OK) {
+        gchar *text;
+
+        if (lbh_get_body_content(info->body, &text) >= 0) {
+            info->download_images = TRUE;
+            webkit_web_view_reload_bypass_cache(info->web_view);
+            webkit_web_view_load_string(info->web_view, text, "text/html",
+                                        libbalsa_message_body_charset
+                                        (info->body), NULL);
+            g_free(text);
+        }
+    }
+
+    gtk_widget_destroy(GTK_WIDGET(info_bar));
+}
+
+static void
+lbh_show_info_bar(LibBalsaWebKitInfo * info)
+{
+    GtkWidget *info_bar_widget;
+    GtkInfoBar *info_bar;
+    GtkWidget *label;
+    GtkWidget *content_area;
+    gchar *text = _("This message part contains images "
+                    "from a remote server. "
+                    "To protect your privacy, "
+                    "Balsa has not downloaded them. "
+                    "You may choose to download them "
+                    "if you trust the server.");
+
+    if (info->has_info_bar)
+        return;
+
+    info_bar_widget =
+        gtk_info_bar_new_with_buttons(_("_Download images"),
+                                     GTK_RESPONSE_OK,
+                                     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+                                     NULL);
+    gtk_box_pack_start(GTK_BOX(info->vbox), info_bar_widget,
+                       FALSE, FALSE, 0);
+
+    info_bar = GTK_INFO_BAR(info_bar_widget);
+
+    label = gtk_label_new(text);
+    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+
+    content_area = gtk_info_bar_get_content_area(info_bar);
+    gtk_container_add(GTK_CONTAINER(content_area), label);
+
+    g_signal_connect(info_bar, "response",
+                     G_CALLBACK(lbh_info_bar_response_cb), info);
+    gtk_info_bar_set_default_response(info_bar, GTK_RESPONSE_CLOSE);
+    gtk_info_bar_set_message_type(info_bar, GTK_MESSAGE_QUESTION);
+
+    info->has_info_bar = TRUE;
+    gtk_widget_show_all(info_bar_widget);
+}
+
 /*
  * Callback for the "resource-request-starting" signal
  *
  * Here we get to disallow all requests involving remote servers, and to
  * handle cid: requests by replacing them with file: requests.
  */
+
 static void
 lbh_resource_request_starting_cb(WebKitWebView         * web_view,
                                  WebKitWebFrame        * frame,
@@ -181,7 +1032,6 @@ lbh_resource_request_starting_cb(WebKitWebView         * web_view,
 
     if (g_ascii_strncasecmp(uri, "cid:", 4)) {
         /* Not a "cid:" request: disable loading. */
-#if GTK_CHECK_VERSION(2, 18, 0)
         static GHashTable *cache = NULL;
 
         if (!cache)
@@ -194,15 +1044,9 @@ lbh_resource_request_starting_cb(WebKitWebView         * web_view,
                                     GINT_TO_POINTER(TRUE));
             } else {
                 webkit_network_request_set_uri(request, "about:blank");
-                gtk_widget_show_all(info->info_bar_widget);
-                gtk_info_bar_set_default_response(GTK_INFO_BAR
-                                                  (info->info_bar_widget),
-                                                  GTK_RESPONSE_CLOSE);
+                lbh_show_info_bar(info);
             }
         }
-#else  /* GTK_CHECK_VERSION(2, 18, 0) */
-        webkit_network_request_set_uri(request, "about:blank");
-#endif /* GTK_CHECK_VERSION(2, 18, 0) */
     } else {
         LibBalsaMessageBody *body;
 
@@ -290,87 +1134,6 @@ lbh_create_web_view_cb(WebKitWebView  * web_view,
     return WEBKIT_WEB_VIEW(widget);
 }
 
-#if GTK_CHECK_VERSION(2, 18, 0)
-/*
- * Make the GtkInfoBar for asking about downloading images
- */
-
-static void
-lbh_info_bar_response_cb(GtkInfoBar * info_bar,
-                         gint response_id, gpointer user_data)
-{
-    LibBalsaWebKitInfo *info = user_data;
-
-    if (response_id == GTK_RESPONSE_OK) {
-        gchar *text;
-
-        if (lbh_get_body_content(info->body, &text) >= 0) {
-            info->download_images = TRUE;
-            webkit_web_view_reload_bypass_cache(info->web_view);
-            webkit_web_view_load_string(info->web_view, text, "text/html",
-                                        libbalsa_message_body_charset
-                                        (info->body), NULL);
-            g_free(text);
-        }
-    }
-
-    gtk_widget_destroy(GTK_WIDGET(info_bar));
-}
-
-static void
-lbh_info_bar_realize_cb(GtkWidget * info_bar,
-                        GtkWidget * text_view)
-{
-    GtkStyle *style = gtk_style_copy(gtk_widget_get_style(info_bar));
-
-    style->base[GTK_STATE_NORMAL] = style->bg[GTK_STATE_NORMAL];
-    gtk_widget_set_style(text_view, style);
-    g_object_unref(style);
-    gtk_widget_hide(info_bar);
-}
-
-static GtkWidget *
-lbh_info_bar_widget(GtkWidget * widget, LibBalsaWebKitInfo * info)
-{
-    GtkWidget *info_bar_widget;
-    GtkInfoBar *info_bar;
-    GtkWidget *text_view_widget;
-    GtkTextView *text_view;
-    GtkWidget *content_area;
-    gchar *text = _("This message part contains images "
-                    "from a remote server. "
-                    "To protect your privacy, "
-                    "Balsa has not downloaded them. "
-                    "You may choose to download them "
-                    "if you trust the server.");
-
-    info_bar_widget =
-        gtk_info_bar_new_with_buttons(_("_Download images"), GTK_RESPONSE_OK,
-                                      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-                                      NULL);
-
-    info_bar = GTK_INFO_BAR(info_bar_widget);
-    gtk_info_bar_set_message_type(info_bar, GTK_MESSAGE_QUESTION);
-    g_signal_connect(info_bar, "response",
-                     G_CALLBACK(lbh_info_bar_response_cb), info);
-
-    text_view_widget = gtk_text_view_new();
-    g_signal_connect(info_bar_widget, "realize",
-                     G_CALLBACK(lbh_info_bar_realize_cb), text_view_widget);
-
-    content_area = gtk_info_bar_get_content_area(info_bar);
-    gtk_container_add(GTK_CONTAINER(content_area), text_view_widget);
-
-    text_view = GTK_TEXT_VIEW(text_view_widget);
-    gtk_text_view_set_wrap_mode(text_view, GTK_WRAP_WORD_CHAR);
-    gtk_text_view_set_editable(text_view, FALSE);
-    gtk_text_buffer_set_text(gtk_text_view_get_buffer(text_view),
-                             text, -1);
-
-    return info_bar_widget;
-}
-#endif /* GTK_CHECK_VERSION(2, 18, 0) */
-
 /* Create a new WebKitWebView widget:
  * text			the HTML source;
  * len			length of text;
@@ -397,23 +1160,17 @@ libbalsa_html_new(LibBalsaMessageBody * body,
     if (len < 0)
         return NULL;
 
-    vbox = gtk_vbox_new(FALSE, 0);
-
-    widget = webkit_web_view_new();
-
     info = g_new(LibBalsaWebKitInfo, 1);
     info->body            = body;
     info->hover_cb        = hover_cb;
     info->clicked_cb      = clicked_cb;
     info->frame           = NULL;
-#if GTK_CHECK_VERSION(2, 18, 0)
     info->download_images = FALSE;
-    info->info_bar_widget = lbh_info_bar_widget(widget, info);
+    info->has_info_bar    = FALSE;
+    info->vbox = vbox     = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
-    gtk_box_pack_start(GTK_BOX(vbox), info->info_bar_widget,
-                       FALSE, FALSE, 0);
-#endif /* GTK_CHECK_VERSION(2, 18, 0) */
-    gtk_box_pack_start(GTK_BOX(vbox), widget, TRUE, TRUE, 0);
+    widget = webkit_web_view_new();
+    gtk_box_pack_end(GTK_BOX(vbox), widget, TRUE, TRUE, 0);
 
     info->web_view = web_view = WEBKIT_WEB_VIEW(widget);
     g_object_set_data(G_OBJECT(vbox), "libbalsa-html-web-view", web_view);
@@ -587,35 +1344,42 @@ lbh_js_object_get_property(JSContextRef  ctx,
  * Search for the text; if text is empty, return TRUE (for consistency
  * with GtkTextIter methods).
  */
-gboolean
-libbalsa_html_search_text(GtkWidget   * widget,
-                          const gchar * text,
-                          gboolean      find_forward,
-                          gboolean      wrap)
+void
+libbalsa_html_search(GtkWidget                * widget,
+                     const gchar              * text,
+                     gboolean                   find_forward,
+                     gboolean                   wrap,
+                     LibBalsaHtmlSearchCallback search_cb,
+                     gpointer                   cb_data)
 {
     WebKitWebView *web_view;
+    gboolean retval;
 
     if (!lbh_get_web_view(widget, &web_view))
-        return FALSE;
+        return;
 
     if (!*text) {
         gchar script[] = "window.getSelection().removeAllRanges()";
 
         lbh_js_run_script(lbh_js_get_global_context(web_view), script);
 
-        return TRUE;
+        (*search_cb)(text, TRUE, cb_data);
+        return;
     }
 
-    return webkit_web_view_search_text(web_view, text,
-                                       FALSE,    /* case-insensitive */
-                                       find_forward, wrap);
+    retval = webkit_web_view_search_text(web_view, text,
+                                         FALSE,    /* case-insensitive */
+                                         find_forward, wrap);
+    (*search_cb)(text, retval, cb_data);
+
+    return;
 }
 
 /*
  * Get the rectangle containing the currently selected text, for
  * scrolling.
  */
-void
+gboolean
 libbalsa_html_get_selection_bounds(GtkWidget    * widget,
                                    GdkRectangle * selection_bounds)
 {
@@ -626,7 +1390,7 @@ libbalsa_html_get_selection_bounds(GtkWidget    * widget,
     JSValueRef value;
 
     if (!lbh_get_web_view(widget, &web_view))
-        return;
+        return FALSE;
 
     ctx = lbh_js_get_global_context(web_view);
     value = lbh_js_run_script(ctx, script);
@@ -647,7 +1411,11 @@ libbalsa_html_get_selection_bounds(GtkWidget    * widget,
             lbh_js_object_get_property(ctx, object, "width");
         selection_bounds->height =
             lbh_js_object_get_property(ctx, object, "height");
+
+        return TRUE;
     }
+
+    return FALSE;
 }
 
 /*
@@ -694,6 +1462,7 @@ libbalsa_html_print(GtkWidget * widget)
     }
 }
 
+#  endif                        /* defined(USE_WEBKIT2) */
 # else                          /* defined(HAVE_WEBKIT) */
 
 /* Common code for both GtkHtml widgets. */
@@ -731,23 +1500,9 @@ lbh_hovering_over_link_cb(GtkWidget   * widget,
         (*info->hover_cb) (uri);
 }
 
-static void
-lbh_size_request_cb(GtkWidget      * widget,
-                    GtkRequisition * requisition,
-                    gpointer         data)
-{
-    GtkLayout *layout = GTK_LAYOUT(widget);
-    GtkAdjustment *adjustment;
+# ifdef HAVE_GTKHTML4
 
-    adjustment = gtk_layout_get_hadjustment(layout);
-    requisition->width = gtk_adjustment_get_upper(adjustment);
-    adjustment = gtk_layout_get_vadjustment(layout);
-    requisition->height = gtk_adjustment_get_upper(adjustment);
-}
-
-# ifdef HAVE_GTKHTML3
-
-/* Code for GtkHTML-3 */
+/* Code for GtkHTML-4 */
 
 #  include <gtkhtml/gtkhtml.h>
 #  include <gtkhtml/gtkhtml-stream.h>
@@ -870,9 +1625,6 @@ libbalsa_html_new(LibBalsaMessageBody * body,
                      G_CALLBACK(libbalsa_html_url_requested),
                      body->message);
 
-    g_signal_connect(widget, "size-request",
-                     G_CALLBACK(lbh_size_request_cb), info);
-
     return widget;
 }
 
@@ -974,7 +1726,7 @@ libbalsa_html_print(GtkWidget * widget)
     g_object_unref(operation);
 }
 
-# else				/* HAVE_GTKHTML3 */
+# else				/* HAVE_GTKHTML4 */
 
 /* Code for GtkHtml-2 */
 
@@ -993,6 +1745,19 @@ libbalsa_html_write_mime_stream(HtmlStream * stream,
     html_stream_close(stream);
 }
 
+static void
+lbh_size_request_cb(GtkWidget      * widget,
+                    GtkRequisition * requisition,
+                    gpointer         data)
+{
+    GtkAdjustment *hadjustment, *vadjustment;
+
+    g_object_get(G_OBJECT(widget), "hadjustment", &hadjustment,
+                                   "vadjustment", &vadjustment, NULL);
+    requisition->width  = gtk_adjustment_get_upper(hadjustment);
+    requisition->height = gtk_adjustment_get_upper(vadjustment);
+}
+
 /* Create a new HtmlView widget:
  * text			the HTML source;
  * len			length of text;
@@ -1130,7 +1895,7 @@ libbalsa_html_print(GtkWidget * widget)
 {
 }
 
-# endif				/* HAVE_GTKHTML3 */
+# endif				/* HAVE_GTKHTML4 */
 
 /* Common code for both widgets. */
 
@@ -1179,17 +1944,21 @@ libbalsa_html_can_search(GtkWidget * widget)
     return FALSE;
 }
 
-gboolean
-libbalsa_html_search_text(GtkWidget * widget, const gchar * text,
-                          gboolean find_forward, gboolean wrap)
+void
+libbalsa_html_search(GtkWidget                * widget,
+                     const gchar              * text,
+                     gboolean                   find_forward,
+                     gboolean                   wrap,
+                     LibBalsaHtmlSearchCallback search_cb,
+                     gpointer                   cb_data)
 {
-    return FALSE;
 }
 
-void
+gboolean
 libbalsa_html_get_selection_bounds(GtkWidget    * widget,
                                    GdkRectangle * selection_bounds)
 {
+    return FALSE;
 }
 
 /*
