@@ -1,6 +1,6 @@
 /* -*-mode:c; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2010 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,9 +33,11 @@
 #include <stdlib.h>
 #include <pthread.h>
 
-#if defined(HAVE_GNOME_KEYRING)
+#if defined(HAVE_LIBSECRET)
+#include <libsecret/secret.h>
+#elif defined(HAVE_GNOME_KEYRING)
 #include <gnome-keyring.h>
-#endif
+#endif                          /* defined(HAVE_LIBSECRET) */
 
 #include "libbalsa.h"
 #include "libbalsa-conf.h"
@@ -65,7 +67,11 @@ struct LibBalsaImapServer_ {
     gboolean offline_mode;
     
 #if defined(BALSA_USE_THREADS)
+#if GLIB_CHECK_VERSION(2, 32, 0)
+    GMutex lock; /* protects the following members */
+#else                           /* GLIB_CHECK_VERSION(2, 32, 0) */
     GMutex *lock; /* protects the following members */
+#endif                          /* GLIB_CHECK_VERSION(2, 32, 0) */
 #endif
     guint used_connections;
     GList *used_handles;
@@ -102,9 +108,15 @@ static gboolean connection_cleanup(gpointer ptr);
 static pthread_mutex_t imap_servers_lock = PTHREAD_MUTEX_INITIALIZER;
 #define LOCK_SERVERS()   pthread_mutex_lock(&imap_servers_lock)
 #define UNLOCK_SERVERS() pthread_mutex_unlock(&imap_servers_lock)
+#if GLIB_CHECK_VERSION(2, 32, 0)
+#define LOCK_SERVER(server)    g_mutex_lock(&((server)->lock))
+#define TRYLOCK_SERVER(server) g_mutex_trylock(&((server)->lock))
+#define UNLOCK_SERVER(server)  g_mutex_unlock(&((server)->lock))
+#else                           /* GLIB_CHECK_VERSION(2, 32, 0) */
 #define LOCK_SERVER(server)    g_mutex_lock((server)->lock)
 #define TRYLOCK_SERVER(server) g_mutex_trylock((server)->lock)
 #define UNLOCK_SERVER(server)  g_mutex_unlock((server)->lock)
+#endif                          /* GLIB_CHECK_VERSION(2, 32, 0) */
 #else
 #define LOCK_SERVERS()
 #define UNLOCK_SERVERS()
@@ -212,7 +224,11 @@ libbalsa_imap_server_init(LibBalsaImapServer * imap_server)
     LIBBALSA_SERVER(imap_server)->protocol = "imap";
     imap_server->key = NULL;
 #if defined(BALSA_USE_THREADS)
+#if GLIB_CHECK_VERSION(2, 32, 0)
+    g_mutex_init(&imap_server->lock);
+#else                           /* GLIB_CHECK_VERSION(2, 32, 0) */
     imap_server->lock = g_mutex_new();
+#endif                          /* GLIB_CHECK_VERSION(2, 32, 0) */
 #endif
     imap_server->max_connections = MAX_CONNECTIONS_PER_SERVER;
     imap_server->used_connections = 0;
@@ -250,7 +266,11 @@ libbalsa_imap_server_finalize(GObject * object)
 #endif
     libbalsa_imap_server_force_disconnect(imap_server);
 #if defined(BALSA_USE_THREADS)
+#if GLIB_CHECK_VERSION(2, 32, 0)
+    g_mutex_clear(&imap_server->lock);
+#else                           /* GLIB_CHECK_VERSION(2, 32, 0) */
     g_mutex_free(imap_server->lock);
+#endif                          /* GLIB_CHECK_VERSION(2, 32, 0) */
 #endif
     g_free(imap_server->key); imap_server->key = NULL;
 
@@ -326,7 +346,7 @@ is_info_cb(ImapMboxHandle *h, ImapResponse rc, const gchar* str, void *arg)
     case IMR_BYE: 
     case IMR_NO: 
         /* IMAP host name + message */
-        fmt = _("%s: %s");  it = LIBBALSA_INFORMATION_MESSAGE; break;
+        fmt = "%s: %s";  it = LIBBALSA_INFORMATION_MESSAGE; break;
     default:
         return;
     }
@@ -438,7 +458,7 @@ lb_imap_server_cleanup(LibBalsaImapServer * imap_server)
 
 static gboolean connection_cleanup(gpointer ptr)
 {
-#ifdef BALSA_USE_THREADS
+#if 0 && defined(BALSA_USE_THREADS)
     pthread_t cleanup_thread;
 #endif                          /*BALSA_USE_THREADS */
     LibBalsaImapServer *imap_server;
@@ -446,7 +466,7 @@ static gboolean connection_cleanup(gpointer ptr)
     g_return_val_if_fail(LIBBALSA_IS_IMAP_SERVER(ptr), FALSE);
 
     imap_server = LIBBALSA_IMAP_SERVER(ptr);
-#ifdef BALSA_USE_THREADS
+#if 0 && defined(BALSA_USE_THREADS)
     pthread_create(&cleanup_thread, NULL,
                    (void *) lb_imap_server_cleanup, imap_server);
     pthread_detach(cleanup_thread);
@@ -551,7 +571,47 @@ libbalsa_imap_server_new_from_config(void)
     if (!server->passwd) {
         server->remember_passwd = libbalsa_conf_get_bool("RememberPasswd=false");
         if(server->remember_passwd) {
-#if defined (HAVE_GNOME_KEYRING)
+#if defined(HAVE_LIBSECRET)
+            GError *err = NULL;
+
+            server->passwd =
+                secret_password_lookup_sync(LIBBALSA_SERVER_SECRET_SCHEMA,
+                                            NULL, &err,
+                                            "protocol", server->protocol,
+                                            "server",   server->host,
+                                            "user",     server->user,
+                                            NULL);
+            if (err) {
+                libbalsa_free_password(server->passwd);
+                server->passwd = NULL;
+                printf(_("Error looking up password for %s@%s: %s\n"),
+                       server->user, server->host, err->message);
+                printf(_("Falling back\n"));
+                g_clear_error(&err);
+                server->passwd =
+                    libbalsa_conf_private_get_string("Password");
+                if (server->passwd != NULL) {
+                    gchar *buff = libbalsa_rot(server->passwd);
+                    libbalsa_free_password(server->passwd);
+                    server->passwd = buff;
+                    secret_password_store_sync
+                        (LIBBALSA_SERVER_SECRET_SCHEMA, NULL,
+                         _("Balsa passwords"), server->passwd, NULL, &err,
+                         "protocol", server->protocol,
+                         "server",   server->host,
+                         "user",     server->user,
+                         NULL);
+                    /* We could in principle clear the password in the
+                     * config file here but we do not for the backward
+                     * compatibility. */
+                    if (err) {
+                        printf(_("Error storing password for %s@%s: %s\n"),
+                               server->user, server->host, err->message);
+                        g_error_free(err);
+                    }
+                }
+            }
+#elif defined (HAVE_GNOME_KEYRING)
 	    GnomeKeyringResult r;
 	    server->passwd = NULL;
 	    r = gnome_keyring_find_password_sync(LIBBALSA_SERVER_KEYRING_SCHEMA,
@@ -589,7 +649,7 @@ libbalsa_imap_server_new_from_config(void)
 		libbalsa_free_password(server->passwd);
 		server->passwd = buff;
 	    }
-#endif
+#endif                          /* defined(HAVE_LIBSECRET) */
 	}
         if(server->passwd && server->passwd[0] == '\0') {
             libbalsa_free_password(server->passwd);
@@ -657,7 +717,6 @@ libbalsa_imap_server_get_handle(LibBalsaImapServer *imap_server, GError **err)
 {
     LibBalsaServer *server = LIBBALSA_SERVER(imap_server);
     struct handle_info *info = NULL;
-    ImapResult rc;
 
     if (!imap_server || imap_server->offline_mode)
         return NULL;
@@ -692,6 +751,8 @@ libbalsa_imap_server_get_handle(LibBalsaImapServer *imap_server, GError **err)
     }
     if (info) {
         if(imap_mbox_is_disconnected(info->handle)) {
+            ImapResult rc;
+
             rc=imap_mbox_handle_connect(info->handle, server->host,
                                         REQ_SSL(server));
             if(rc != IMAP_SUCCESS) {
@@ -707,6 +768,7 @@ libbalsa_imap_server_get_handle(LibBalsaImapServer *imap_server, GError **err)
     }
     UNLOCK_SERVER(imap_server);
 
+    /* cppcheck-suppress nullPointer */
     return info ? info->handle : NULL;
 }
 
@@ -730,7 +792,6 @@ libbalsa_imap_server_get_handle_with_user(LibBalsaImapServer *imap_server,
 {
     LibBalsaServer *server = LIBBALSA_SERVER(imap_server);
     struct handle_info *info = NULL;
-    ImapResult rc;
 
     if (imap_server->offline_mode)
         return NULL;
@@ -788,6 +849,8 @@ libbalsa_imap_server_get_handle_with_user(LibBalsaImapServer *imap_server,
     }
 
     if (imap_mbox_is_disconnected(info->handle)) {
+        ImapResult rc;
+
         rc=imap_mbox_handle_connect(info->handle, server->host,
                                     REQ_SSL(server));
         if(rc != IMAP_SUCCESS) {
