@@ -111,6 +111,20 @@ struct PopHandle_ {
 };
 #define pop_can_do(pop, cap) ((pop)->capabilities[cap])
 
+/*
+ * IMAP_ERROR domain for g_set_error()
+ */
+GQuark
+pop_imap_error_quark(void)
+{
+    static GQuark quark;
+
+    if (!quark)
+        quark = g_quark_from_static_string("imap-pop3-error-quark");
+
+    return quark;
+}
+
 PopHandle *
 pop_new(void)
 {
@@ -183,8 +197,10 @@ pop_check_status(PopHandle *pop, GError **err)
      
   if(strncmp(buf, "+OK", 3) == 0)
     res = TRUE;
+  /*
   else if(strncmp(buf, "-ERR", 4) == 0)
     res = FALSE;
+    */
   else
     res = FALSE;
 
@@ -459,7 +475,7 @@ parse_list_response(PopHandle *pop, char *line, ssize_t sz, GError **err)
       }
       if(line[0]=='.' && (line[1] == '\r' || line[1] == '\n'))
         break;
-      if( sscanf(line, "%u%u", &msg, &msg_size) < 2 ) {
+      if( sscanf(line, "%10u%10u", &msg, &msg_size) < 2 ) {
         g_set_error(err, IMAP_ERROR, IMAP_POP_PROTOCOL_ERROR,
                     "Server %s did not response correctly to LIST: %s",
                     pop->host, line);
@@ -634,10 +650,13 @@ pop_fetch_message_s(PopHandle *pop, unsigned msgno,
   resp = pop_check_status(pop, err);
   /* do here the fetch */
   if(resp) {
+    /* whether next line will be a continuation line. */
+    gboolean continuation_line = FALSE;
     while( sio_gets(pop->sio, line, sizeof(line)) &&
-           strcmp(line, ".\r\n") ) {
+           (continuation_line || strcmp(line, ".\r\n")) ) {
       char *arg = line[0] == '.' ? line+1 : line;
       unsigned len = strlen(arg);
+      continuation_line = (len >= POP_LINE_LEN-1);
       if(pop->filter_cr && len>=2 && arg[len-2] == '\r') 
         arg[(--len)-1] = '\n';
       if(resp) 
@@ -727,10 +746,13 @@ pop_complete_retr(PopHandle *pop, PopAsyncCb cb, void *arg)
     cb(rc, arg, &err);
   if(resp) { /* same code as in fetch_message() */
     char * str;
+    /* whether next line will be a continuation line. */
+    gboolean continuation_line = FALSE;
     while( (str = sio_gets(pop->sio, line, sizeof(line))) &&
-           strcmp(line, ".\r\n") ) {
+           (continuation_line || strcmp(line, ".\r\n")) ) {
       char *buf = line[0] == '.' ? line+1 : line;
       unsigned len = strlen(buf);
+      continuation_line = (len >= POP_LINE_LEN-1);
       if(pop->filter_cr && len>=2 && buf[len-2] == '\r') 
         buf[(--len)-1] = '\n';
       if(cb) 
