@@ -118,7 +118,9 @@ lb_gpgme_passphrase(void *hook, const gchar * uid_hint,
 	passwd = apd.res;
     }
 #else
-    passwd = get_passphrase_real(context, uid_hint, prev_was_bad);
+    passwd =
+        get_passphrase_real(uid_hint, passphrase_info, prev_was_bad,
+                            parent);
 #endif				/* BALSA_USE_THREADS */
 
     if (!passwd) {
@@ -165,7 +167,7 @@ lb_gpgme_select_key(const gchar * user_name, gboolean secret, GList * keys,
 #if HAVE_MACOSX_DESKTOP
     libbalsa_macosx_menu_for_parent(dialog, parent);
 #endif
-    vbox = gtk_vbox_new(FALSE, 12);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 12);
     gtk_container_add(GTK_CONTAINER
 		      (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
 		      vbox);
@@ -348,13 +350,13 @@ get_passphrase_real(const gchar * uid_hint, const gchar * passphrase_info,
 #if HAVE_MACOSX_DESKTOP
     libbalsa_macosx_menu_for_parent(dialog, parent);
 #endif
-    hbox = gtk_hbox_new(FALSE, 12);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), 12);
     gtk_container_add(GTK_CONTAINER
 		      (gtk_dialog_get_content_area(GTK_DIALOG(dialog))),
 		      hbox);
 
-    vbox = gtk_vbox_new(FALSE, 12);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 12);
     gtk_container_add(GTK_CONTAINER(hbox), vbox);
     if (!padlock_keyhole)
 	padlock_keyhole =
@@ -362,7 +364,7 @@ get_passphrase_real(const gchar * uid_hint, const gchar * passphrase_info,
     gtk_box_pack_start(GTK_BOX(vbox),
 		       gtk_image_new_from_pixbuf(padlock_keyhole), FALSE,
 		       FALSE, 0);
-    vbox = gtk_vbox_new(FALSE, 12);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 12);
     gtk_container_add(GTK_CONTAINER(hbox), vbox);
     if (prev_was_bad)
 	prompt =
