@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -47,9 +47,6 @@
 #endif                          /* HAVE_COMPFACE */
 
 #if HAVE_GTKSOURCEVIEW
-#ifndef G_CONST_RETURN
-#  define G_CONST_RETURN const
-#endif
 #include <gtksourceview/gtksourceview.h>
 #include <gtksourceview/gtksourcebuffer.h>
 /* note GtkSourceview 1 and 2 have a slightly different API */
@@ -70,7 +67,6 @@
 
 #ifdef BALSA_USE_THREADS
 static pthread_t main_thread_id;
-static pthread_t libbalsa_threads_id;
 #endif
 
 
@@ -95,9 +91,11 @@ libbalsa_init(LibBalsaInformationFunc information_callback)
 #endif
 
 #ifdef BALSA_USE_THREADS
+#if !GLIB_CHECK_VERSION(2, 35, 0)
     if (!g_thread_supported()) {
 	g_error("Threads have not been initialised.");
     }
+#endif                          /* !GLIB_CHECK_VERSION(2, 35, 0) */
     main_thread_id = pthread_self();
 #endif
 
@@ -217,7 +215,6 @@ libbalsa_guess_email_address(void)
 gchar *
 libbalsa_guess_mail_spool(void)
 {
-    int i;
     gchar *env;
     gchar *spool;
     static const gchar *guesses[] = {
@@ -232,6 +229,8 @@ libbalsa_guess_mail_spool(void)
 	return g_strdup(env);
 
     if ((env = getenv("USER")) != NULL) {
+        int i;
+
 	for (i = 0; guesses[i] != NULL; i++) {
 	    spool = g_strconcat(guesses[i], env, NULL);
 
@@ -380,11 +379,11 @@ static char*
 asn1time_to_string(ASN1_UTCTIME *tm)
 {
     char buf[64];
-    int cnt;
     BIO *bio  = BIO_new(BIO_s_mem());
     strncpy(buf, _("Invalid date"), sizeof(buf)); buf[sizeof(buf)-1]='\0';
 
     if(ASN1_TIME_print(bio, tm)) {
+        int cnt;
         cnt = BIO_read(bio, buf, sizeof(buf)-1);
         buf[cnt] = '\0';
     }
@@ -396,12 +395,14 @@ static char*
 x509_get_part (char *line, const char *ndx)
 {
     static char ret[256];
-    char *c, *c2;
-    
+    char *c;
+
     strncpy (ret, _("Unknown"), sizeof (ret)); ret[sizeof(ret)-1]='\0';
-    
+
     c = strstr(line, ndx);
     if (c) {
+        char *c2;
+
         c += strlen (ndx);
         c2 = strchr (c, '/');
         if (c2)
@@ -410,7 +411,7 @@ x509_get_part (char *line, const char *ndx)
         if (c2)
             *c2 = '/';
     }
-    
+
     return ret;
 }
 static void
@@ -576,6 +577,16 @@ ask_cert_real(void *data)
     g_string_append(str, c); g_free(c);
     g_free(valid_from); g_free(valid_until);
 
+    /* This string uses markup, so we must replace "&" with "&amp;" */
+    c = str->str;
+    while ((c = strchr(c, '&'))) {
+        gssize pos;
+
+        pos = (c - str->str) + 1;
+        g_string_insert(str, pos, "amp;");
+        c = str->str + pos;
+    }
+
     dialog = gtk_dialog_new_with_buttons(_("SSL/TLS certificate"), NULL,
                                          GTK_DIALOG_MODAL,
                                          _("_Accept Once"), 0,
@@ -684,32 +695,12 @@ void
 libbalsa_lock_mailbox(LibBalsaMailbox * mailbox)
 {
     pthread_t thread_id = pthread_self();
-    gint count = 0;
-
-    if (thread_id == libbalsa_threads_id
-        && thread_id == mailbox->thread_id
-        && mailbox->lock > 0) {
-        /* We already have both locks, so we'll just hold on to both of
-         * them. */
-        ++mailbox->lock;
-#if LIBBALSA_DEBUG_THREADS
-        g_message("Avoided temporary gdk_threads_leave!!!");
-#endif                          /* LIBBALSA_DEBUG_THREADS */
-        return;
-    }
-
-    while (thread_id == libbalsa_threads_id) {
-        ++count;
-#if LIBBALSA_DEBUG_THREADS
-        g_message("Temporary gdk_threads_leave!!!");
-#endif                          /* LIBBALSA_DEBUG_THREADS */
-        gdk_threads_leave();
-    }
 
     pthread_mutex_lock(&mailbox_mutex);
 
-    while (mailbox->lock && mailbox->thread_id != thread_id)
-        pthread_cond_wait(&mailbox_cond, &mailbox_mutex);
+    if (mailbox->thread_id && mailbox->thread_id != thread_id)
+        while (mailbox->lock)
+            pthread_cond_wait(&mailbox_cond, &mailbox_mutex);
 
     /* We'll assume that no-one would destroy a mailbox while we've been
      * trying to lock it. If they have, we have larger problems than
@@ -718,13 +709,6 @@ libbalsa_lock_mailbox(LibBalsaMailbox * mailbox)
     mailbox->thread_id = thread_id;
 
     pthread_mutex_unlock(&mailbox_mutex);
-
-    while (--count >= 0) {
-        gdk_threads_enter();
-#if LIBBALSA_DEBUG_THREADS
-        g_message("...and gdk_threads_enter!!!");
-#endif                          /* LIBBALSA_DEBUG_THREADS */
-    }
 }
 
 void
@@ -743,72 +727,13 @@ libbalsa_unlock_mailbox(LibBalsaMailbox * mailbox)
     }
 
     if(--mailbox->lock == 0) {
-        pthread_cond_broadcast(&mailbox_cond);
         mailbox->thread_id = 0;
+        pthread_cond_broadcast(&mailbox_cond);
     }
 
     pthread_mutex_unlock(&mailbox_mutex);
 }
 
-/* Recursive mutex for gdk_threads_{enter,leave}. */
-static pthread_mutex_t libbalsa_threads_mutex;
-static guint libbalsa_threads_lock;
-
-static void
-libbalsa_threads_enter(void)
-{
-    pthread_t self;
-
-    self = pthread_self();
-
-    if (self != libbalsa_threads_id) {
-        pthread_mutex_lock(&libbalsa_threads_mutex);
-        libbalsa_threads_id = self;
-    }
-    ++libbalsa_threads_lock;
-}
-
-static void
-libbalsa_threads_leave(void)
-{
-    pthread_t self;
-
-    self = pthread_self();
-
-    if (libbalsa_threads_lock == 0 || self != libbalsa_threads_id) {
-        g_warning("%s: Not holding gdk lock!!!", __func__);
-	return;
-    }
-
-    if (--libbalsa_threads_lock == 0) {
-	if (self != main_thread_id)
-	    gdk_display_flush(gdk_display_get_default());
-        libbalsa_threads_id = 0;
-        pthread_mutex_unlock(&libbalsa_threads_mutex);
-    }
-}
-
-void
-libbalsa_threads_init(void)
-{
-    pthread_mutex_init(&libbalsa_threads_mutex, NULL);
-    gdk_threads_set_lock_functions(G_CALLBACK(libbalsa_threads_enter),
-                                   G_CALLBACK(libbalsa_threads_leave));
-}
-
-void
-libbalsa_threads_destroy(void)
-{
-    pthread_mutex_destroy(&libbalsa_threads_mutex);
-}
-
-gboolean
-libbalsa_threads_has_lock(void)
-{
-    return libbalsa_threads_lock > 0
-        && libbalsa_threads_id == pthread_self();
-}
-
 #endif				/* BALSA_USE_THREADS */
 
 /* Initialized by the front end. */
@@ -890,16 +815,6 @@ libbalsa_get_image_from_face_header(const gchar * content, GError ** err)
     return image;
 }
 
-
-GQuark
-libbalsa_image_error_quark(void)
-{
-    static GQuark quark = 0;
-    if (quark == 0)
-        quark = g_quark_from_static_string("libbalsa-image-error-quark");
-    return quark;
-}
-
 #if HAVE_COMPFACE
 GtkWidget *
 libbalsa_get_image_from_x_face_header(const gchar * content, GError ** err)
@@ -933,7 +848,7 @@ libbalsa_get_image_from_x_face_header(const gchar * content, GError ** err)
         gint j, k;
         guchar *q;
 
-        if (sscanf(p, "%x,%x,%x,", &x[0], &x[1], &x[2]) != 3) {
+        if (sscanf(p, "%8x,%8x,%8x,", &x[0], &x[1], &x[2]) != 3) {
             g_set_error(err, LIBBALSA_IMAGE_ERROR,
                         LIBBALSA_IMAGE_ERROR_BAD_DATA,
                         /* Translators: please do not translate Face. */
@@ -961,90 +876,11 @@ libbalsa_get_image_from_x_face_header(const gchar * content, GError ** err)
 
 #if HAVE_GTKSOURCEVIEW
 GtkWidget *
-libbalsa_source_view_new(gboolean highlight_phrases, GdkColor *q_colour)
+libbalsa_source_view_new(gboolean highlight_phrases)
 {
     GtkSourceBuffer *sbuffer;
     GtkWidget *sview;
 
-#if (HAVE_GTKSOURCEVIEW == 1)
-
-    GtkTextTag * text_tag;
-    GtkSourceTagStyle *tag_style;
-    GtkSourceTagTable *tag_table;
-    GSList *tag_list;
-
-    /* create the tag table */
-    tag_list = NULL;
-    tag_table = gtk_source_tag_table_new();
-
-    /* add highlighting for quoted text if requested */
-    if (q_colour) {
-	int k;
-
-	for (k = 1; k <= 9; k++) {
-	    gchar * tag_id;
-	    const gchar * pattern;
-            gchar *tmp = NULL;
-
-	    tag_id = g_strdup_printf("Quote-%d", k);
-	    if (k == 1)
-		pattern = "^> *($|[^ |>:}#\n])";
-	    else
-		pattern = tmp =
-                    g_strdup_printf("^(> *){%d}($|[^ |>:}#\n])", k);
-	    text_tag = gtk_line_comment_tag_new(tag_id, tag_id, pattern);
-	    g_free(tmp);
-	    g_free(tag_id);
-	    tag_style = gtk_source_tag_style_new();
-	    tag_style->mask = GTK_SOURCE_TAG_STYLE_USE_FOREGROUND;
-	    tag_style->foreground = q_colour[(k - 1) & 1];
-	    gtk_source_tag_set_style(GTK_SOURCE_TAG(text_tag), tag_style);
-	    gtk_source_tag_style_free(tag_style);
-	    tag_list = g_slist_prepend(tag_list, text_tag);
-	}
-    }
-
-    /* if requested create the patterns for bold, italic and underline */
-    if (highlight_phrases) {
-	text_tag = gtk_pattern_tag_new("Bold", "Bold",
-				       "(^|[[:space:]])\\*[[:alnum:]][^*\n]*[[:alnum:]]\\*");
-	tag_style = gtk_source_tag_style_new();
-	tag_style->bold = TRUE;
-	gtk_source_tag_set_style(GTK_SOURCE_TAG(text_tag), tag_style);
-	gtk_source_tag_style_free(tag_style);
-	tag_list = g_slist_prepend(tag_list, text_tag);
-
-	text_tag = gtk_pattern_tag_new("Italic", "Italic",
-				       "(^|[[:space:]])/[[:alnum:]][^/\n]*[[:alnum:]]/");
-	tag_style = gtk_source_tag_style_new();
-	tag_style->italic = TRUE;
-	gtk_source_tag_set_style(GTK_SOURCE_TAG(text_tag), tag_style);
-	gtk_source_tag_style_free(tag_style);
-	tag_list = g_slist_prepend(tag_list, text_tag);
-
-	text_tag = gtk_pattern_tag_new("Underline", "Underline",
-				       "(^|[[:space:]])_[[:alnum:]][^_\n]*[[:alnum:]]_");
-	tag_style = gtk_source_tag_style_new();
-	tag_style->underline = TRUE;
-	gtk_source_tag_set_style(GTK_SOURCE_TAG(text_tag), tag_style);
-	gtk_source_tag_style_free(tag_style);
-	tag_list = g_slist_prepend(tag_list, text_tag);
-    }
-
-    /* add tags to the table if present */
-    if (tag_list) {
-	gtk_source_tag_table_add_tags(tag_table, tag_list);
-	g_slist_foreach(tag_list, (GFunc)g_object_unref, NULL);
-	g_slist_free(tag_list);
-    }
-
-    /* create the source buffer */
-    sbuffer = gtk_source_buffer_new(tag_table);
-    g_object_unref(tag_table);
-    gtk_source_buffer_set_highlight(sbuffer, highlight_phrases || q_colour);
-    gtk_source_buffer_set_check_brackets(sbuffer, FALSE);
-
-#else /* (HAVE_GTKSOURCEVIEW == 1) */
 
     static GtkSourceLanguageManager * lm = NULL;
     static GtkSourceStyleScheme * scheme = NULL;
@@ -1095,8 +931,6 @@ libbalsa_source_view_new(gboolean highlight_phrases, GdkColor *q_colour)
     gtk_source_buffer_set_highlight_syntax(sbuffer, TRUE);
     gtk_source_buffer_set_highlight_matching_brackets(sbuffer, FALSE);
 
-#endif /* (HAVE_GTKSOURCEVIEW == 1) */
-
     /* create & return the source view */
     sview = gtk_source_view_new_with_buffer(sbuffer);
     g_object_unref(sbuffer);
@@ -1105,3 +939,33 @@ libbalsa_source_view_new(gboolean highlight_phrases, GdkColor *q_colour)
 }
 #endif  /* HAVE_GTKSOURCEVIEW */
 
+/*
+ * Error domains for GError:
+ */
+
+GQuark
+libbalsa_scanner_error_quark(void)
+{
+    static GQuark quark = 0;
+    if (quark == 0)
+        quark = g_quark_from_static_string("libbalsa-scanner-error-quark");
+    return quark;
+}
+
+GQuark
+libbalsa_mailbox_error_quark(void)
+{
+    static GQuark quark = 0;
+    if (quark == 0)
+        quark = g_quark_from_static_string("libbalsa-mailbox-error-quark");
+    return quark;
+}
+
+GQuark
+libbalsa_image_error_quark(void)
+{
+    static GQuark quark = 0;
+    if (quark == 0)
+        quark = g_quark_from_static_string("libbalsa-image-error-quark");
+    return quark;
+}
