@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -78,10 +78,10 @@ typedef struct _LibbalsaVfs LibbalsaVfs;
 /*
  * Error domains for GError:
  */
-enum {
-    LIBBALSA_SCANNER_ERROR,
-    LIBBALSA_MAILBOX_ERROR
-};
+GQuark libbalsa_scanner_error_quark(void);
+#define LIBBALSA_SCANNER_ERROR libbalsa_scanner_error_quark()
+GQuark libbalsa_mailbox_error_quark(void);
+#define LIBBALSA_MAILBOX_ERROR libbalsa_mailbox_error_quark()
 
 /*
  * Error codes for GError:
@@ -143,13 +143,18 @@ gboolean libbalsa_abort_on_timeout(const char *host);
 #include <pthread.h>
 pthread_t libbalsa_get_main_thread(void);
 gboolean libbalsa_am_i_subthread(void);
-void libbalsa_threads_init(void);
-void libbalsa_threads_destroy(void);
-gboolean libbalsa_threads_has_lock(void);
 #else
 #define libbalsa_am_i_subthread() FALSE
-#define libbalsa_threads_has_lock() TRUE
 #endif /* BALSA_USE_THREADS */
+#if defined(BALSA_DEBUG_THREADS)
+#define gdk_threads_enter()                       \
+    do if (libbalsa_am_i_subthread())             \
+        g_warning("%s: sub-thread!\n", __func__); \
+    while (0)
+#else
+#define gdk_threads_enter()
+#endif
+#define gdk_threads_leave()
 void libbalsa_message(const char *fmt, ...)
 #ifdef __GNUC__
     __attribute__ ((format (printf, 1, 2)))
@@ -205,7 +210,7 @@ enum LibBalsaImageError {
 };
 
 #if HAVE_GTKSOURCEVIEW
-GtkWidget *libbalsa_source_view_new(gboolean highlight_phrases, GdkColor *q_colour);
+GtkWidget *libbalsa_source_view_new(gboolean highlight_phrases);
 #endif                          /* HAVE_GTKSOURCEVIEW */
 
 #endif                          /* __LIBBALSA_H__ */
