@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -45,6 +45,7 @@ static void libbalsa_mailbox_local_class_init(LibBalsaMailboxLocalClass *klass);
 static void libbalsa_mailbox_local_init(LibBalsaMailboxLocal * mailbox);
 static void libbalsa_mailbox_local_finalize(GObject * object);
 
+static void libbalsa_mailbox_local_changed(LibBalsaMailbox * mailbox);
 static void libbalsa_mailbox_local_save_config(LibBalsaMailbox * mailbox,
 					       const gchar * prefix);
 static void libbalsa_mailbox_local_load_config(LibBalsaMailbox * mailbox,
@@ -142,6 +143,8 @@ libbalsa_mailbox_local_class_init(LibBalsaMailboxLocalClass * klass)
 
     object_class->finalize = libbalsa_mailbox_local_finalize;
 
+    libbalsa_mailbox_class->changed =
+	libbalsa_mailbox_local_changed;
     libbalsa_mailbox_class->save_config =
 	libbalsa_mailbox_local_save_config;
     libbalsa_mailbox_class->load_config =
@@ -315,6 +318,8 @@ libbalsa_mailbox_local_load_message(LibBalsaMailboxLocal * local,
     LibBalsaMailbox *mbx = LIBBALSA_MAILBOX(local);
     gboolean match;
 
+    msg_info->loaded = TRUE;
+
     if ((msg_info->flags & LIBBALSA_MESSAGE_FLAG_NEW)
         && !(msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED)) {
         mbx->unread_messages++;
@@ -394,10 +399,23 @@ libbalsa_mailbox_local_finalize(GObject * object)
 	ml->threading_info = NULL;
     }
 
+    if (ml->load_messages_id) {
+        g_source_remove(ml->load_messages_id);
+        ml->load_messages_id = 0;
+    }
+
     if (G_OBJECT_CLASS(parent_class)->finalize)
 	G_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
+static void lbm_local_queue_save_tree(LibBalsaMailboxLocal * local);
+
+static void
+libbalsa_mailbox_local_changed(LibBalsaMailbox * mailbox)
+{
+    lbm_local_queue_save_tree(LIBBALSA_MAILBOX_LOCAL(mailbox));
+}
+
 static void
 libbalsa_mailbox_local_save_config(LibBalsaMailbox * mailbox,
 				   const gchar * prefix)
@@ -558,6 +576,8 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
     GNode *parent, *sibling;
     LibBalsaMailboxLocalTreeInfo *info;
     guint8 *seen;
+    LibBalsaMailboxLocalMessageInfo *(*get_info) (LibBalsaMailboxLocal *,
+                                                  guint);
 
     filename = lbm_local_get_cache_filename(local);
     name = mailbox->name ? g_strdup(mailbox->name) :
@@ -609,7 +629,9 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
     seen = g_new0(guint8, *total);
     parent = mailbox->msg_tree;
     sibling = NULL;
+    get_info = LIBBALSA_MAILBOX_LOCAL_GET_CLASS(local)->get_info;
     while (++info < (LibBalsaMailboxLocalTreeInfo *) (contents + length)) {
+        LibBalsaMailboxLocalMessageInfo *msg_info;
         if (info->msgno == 0 || info->msgno > *total
             || seen[info->msgno - 1]) {
             libbalsa_information(LIBBALSA_INFORMATION_DEBUG,
@@ -649,6 +671,10 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
         }
         libbalsa_mailbox_msgno_inserted(mailbox, info->msgno,
                                         parent, &sibling);
+
+        msg_info = get_info(local, info->msgno);
+        msg_info->loaded = TRUE;
+
         if (libbalsa_mailbox_msgno_has_flags(mailbox, info->msgno,
                                              LIBBALSA_MESSAGE_FLAG_NEW,
                                              LIBBALSA_MESSAGE_FLAG_DELETED))
@@ -672,20 +698,21 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
 static void
 lbm_local_save_tree_real(LibBalsaMailboxLocal * local)
 {
-    libbalsa_lock_mailbox(LIBBALSA_MAILBOX(local));
+    LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(local);
 
-    if (MAILBOX_OPEN(LIBBALSA_MAILBOX(local)))
+    libbalsa_lock_mailbox(mailbox);
+
+    if (MAILBOX_OPEN(mailbox) && mailbox->msg_tree_changed)
         lbm_local_save_tree(local);
     local->save_tree_id = 0;
-    g_object_unref(local);
 
-    libbalsa_unlock_mailbox(LIBBALSA_MAILBOX(local));
+    libbalsa_unlock_mailbox(mailbox);
 }
 
 static gboolean
 lbm_local_save_tree_idle(LibBalsaMailboxLocal * local)
 {
-#ifdef BALSA_USE_THREADS
+#if 0 && defined(BALSA_USE_THREADS)
     pthread_t save_tree_thread;
 
     pthread_create(&save_tree_thread, NULL,
@@ -701,12 +728,11 @@ lbm_local_save_tree_idle(LibBalsaMailboxLocal * local)
 static void
 lbm_local_queue_save_tree(LibBalsaMailboxLocal * local)
 {
-    if (LIBBALSA_MAILBOX(local)->msg_tree_changed
-        && !local->save_tree_id) {
-        g_object_ref(local);
+    libbalsa_lock_mailbox((LibBalsaMailbox *) local);
+    if (!local->save_tree_id)
         local->save_tree_id =
             g_idle_add((GSourceFunc) lbm_local_save_tree_idle, local);
-    }
+    libbalsa_unlock_mailbox((LibBalsaMailbox *) local);
 }
 
 /* 
@@ -1010,10 +1036,10 @@ libbalsa_mailbox_local_cache_message(LibBalsaMailboxLocal * local,
         info->sender = g_strdup("");
 }
 
-void
-libbalsa_mailbox_local_load_messages(LibBalsaMailbox *mailbox,
-                                     guint msgno)
+static gboolean
+lbml_load_messages_idle_cb(LibBalsaMailbox * mailbox)
 {
+    guint msgno;
     guint new_messages;
     LibBalsaMailboxLocal *local;
     guint lastno;
@@ -1021,18 +1047,19 @@ libbalsa_mailbox_local_load_messages(LibBalsaMailbox *mailbox,
     LibBalsaMailboxLocalMessageInfo *(*get_info) (LibBalsaMailboxLocal *,
                                                   guint);
 
-    g_return_if_fail(LIBBALSA_IS_MAILBOX_LOCAL(mailbox));
-
+    libbalsa_lock_mailbox(mailbox);
     gdk_threads_enter();
 
     if (!mailbox->msg_tree) {
 	/* Mailbox is closed, or no view has been created. */
         gdk_threads_leave();
-	return;
+        libbalsa_unlock_mailbox(mailbox);
+	return FALSE;
     }
 
     local = (LibBalsaMailboxLocal *) mailbox;
     lastno = libbalsa_mailbox_total_messages(mailbox);
+    msgno = local->msgno;
     new_messages = lastno - msgno;
     lastn = g_node_last_child(mailbox->msg_tree);
     get_info = LIBBALSA_MAILBOX_LOCAL_GET_CLASS(local)->get_info;
@@ -1054,6 +1081,28 @@ libbalsa_mailbox_local_load_messages(LibBalsaMailbox *mailbox,
 						  mailbox->
 						  unread_messages > 0);
     }
+
+    local->load_messages_id = 0;
+    libbalsa_unlock_mailbox(mailbox);
+    return FALSE;
+}
+
+void
+libbalsa_mailbox_local_load_messages(LibBalsaMailbox *mailbox,
+                                     guint msgno)
+{
+    LibBalsaMailboxLocal *local;
+
+    g_return_if_fail(LIBBALSA_IS_MAILBOX_LOCAL(mailbox));
+
+    local = (LibBalsaMailboxLocal *) mailbox;
+    libbalsa_lock_mailbox(mailbox);
+    if (!local->load_messages_id) {
+        local->msgno = msgno;
+        local->load_messages_id =
+            g_idle_add((GSourceFunc) lbml_load_messages_idle_cb, mailbox);
+    }
+    libbalsa_unlock_mailbox(mailbox);
 }
 
 /*
@@ -1071,6 +1120,37 @@ libbalsa_mailbox_local_set_threading_info(LibBalsaMailboxLocal * local)
         local->threading_info = g_ptr_array_new();
 }
 
+static void
+lbml_set_threading(LibBalsaMailbox * mailbox,
+                   LibBalsaMailboxThreadingType thread_type)
+{
+    switch (thread_type) {
+    case LB_MAILBOX_THREADING_JWZ:
+        lbml_threading_jwz(mailbox);
+        break;
+    case LB_MAILBOX_THREADING_FLAT:
+    case LB_MAILBOX_THREADING_SIMPLE:
+        lbml_threading_simple(mailbox, thread_type);
+        break;
+    }
+}
+
+#ifdef BALSA_USE_THREADS
+typedef struct {
+    LibBalsaMailbox *mailbox;
+    LibBalsaMailboxThreadingType thread_type;
+} LbmlSetThreadingInfo;
+
+static gboolean
+lbml_set_threading_idle_cb(LbmlSetThreadingInfo * info)
+{
+    lbml_set_threading(info->mailbox, info->thread_type);
+    g_object_unref(info->mailbox);
+    g_slice_free(LbmlSetThreadingInfo, info);
+    return FALSE;
+}
+#endif                          /* BALSA_USE_THREADS */
+
 static void
 libbalsa_mailbox_local_set_threading(LibBalsaMailbox * mailbox,
                                      LibBalsaMailboxThreadingType
@@ -1113,17 +1193,22 @@ libbalsa_mailbox_local_set_threading(LibBalsaMailbox * mailbox,
             return;
     }
 
-    gdk_threads_enter();
-    switch (thread_type) {
-    case LB_MAILBOX_THREADING_JWZ:
-        lbml_threading_jwz(mailbox);
-        break;
-    case LB_MAILBOX_THREADING_FLAT:
-    case LB_MAILBOX_THREADING_SIMPLE:
-        lbml_threading_simple(mailbox, thread_type);
-        break;
+#ifdef BALSA_USE_THREADS
+    if (libbalsa_am_i_subthread()) {
+        LbmlSetThreadingInfo *info;
+
+        info = g_slice_new(LbmlSetThreadingInfo);
+        info->mailbox = g_object_ref(mailbox);
+        info->thread_type = thread_type;
+        gdk_threads_add_idle((GSourceFunc) lbml_set_threading_idle_cb, info);
+    } else {
+        gdk_threads_enter();
+        lbml_set_threading(mailbox, thread_type);
+        gdk_threads_leave();
     }
-    gdk_threads_leave();
+#else                           /* BALSA_USE_THREADS */
+    lbml_set_threading(mailbox, thread_type);
+#endif                          /* BALSA_USE_THREADS */
 #if defined(DEBUG_LOADING_AND_THREADING)
     printf("after threading time=%lu\n", (unsigned long) time(NULL));
 #endif
@@ -1177,7 +1262,7 @@ lbm_local_update_view_filter(LibBalsaMailbox * mailbox,
                                        ((gdouble) total));
     }
     libbalsa_progress_set_text(&progress, NULL, 0);
-    libbalsa_mailbox_search_iter_free(iter_view);
+    libbalsa_mailbox_search_iter_unref(iter_view);
 
     /* If this is not a flags-only filter, the new mailbox tree is
      * temporary, so we don't want to save it. */
@@ -1249,6 +1334,7 @@ libbalsa_mailbox_local_prepare_threading(LibBalsaMailbox * mailbox,
     LibBalsaProgress progress = LIBBALSA_PROGRESS_INIT;
     gboolean retval = TRUE;
 
+    libbalsa_lock_mailbox(mailbox);
     libbalsa_mailbox_local_set_threading_info(local);
 
     text = g_strdup_printf(_("Preparing %s"), mailbox->name);
@@ -1284,6 +1370,7 @@ libbalsa_mailbox_local_prepare_threading(LibBalsaMailbox * mailbox,
                 g_idle_add((GSourceFunc) lbm_local_thread_idle, local);
         }
     }
+    libbalsa_unlock_mailbox(mailbox);
 
     return retval;
 }
@@ -2081,7 +2168,7 @@ lbm_local_sync_real(LibBalsaMailboxLocal * local)
 static gboolean
 lbm_local_sync_idle(LibBalsaMailboxLocal * local)
 {
-#ifdef BALSA_USE_THREADS
+#if 0 && defined(BALSA_USE_THREADS)
     pthread_t sync_thread;
 
     pthread_create(&sync_thread, NULL, (void *) lbm_local_sync_real, local);
@@ -2143,7 +2230,6 @@ libbalsa_mailbox_local_messages_change_flags(LibBalsaMailbox * mailbox,
         guint msgno = g_array_index(msgnos, guint, i);
         LibBalsaMailboxLocalMessageInfo *msg_info;
         LibBalsaMessageFlag old_flags;
-        gboolean was_unread_undeleted, is_unread_undeleted;
 
         if (!(msgno > 0
               && msgno <= libbalsa_mailbox_total_messages(mailbox))) {
@@ -2165,12 +2251,18 @@ libbalsa_mailbox_local_messages_change_flags(LibBalsaMailbox * mailbox,
 
         libbalsa_mailbox_index_set_flags(mailbox, msgno, msg_info->flags);
 
-        was_unread_undeleted = (old_flags & LIBBALSA_MESSAGE_FLAG_NEW)
-            && !(old_flags & LIBBALSA_MESSAGE_FLAG_DELETED);
-        is_unread_undeleted = (msg_info->flags & LIBBALSA_MESSAGE_FLAG_NEW)
-            && !(msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED);
-        mailbox->unread_messages +=
-            is_unread_undeleted - was_unread_undeleted;
+        if (msg_info->loaded) {
+            gboolean was_unread_undeleted, is_unread_undeleted;
+
+            was_unread_undeleted =
+                (old_flags & LIBBALSA_MESSAGE_FLAG_NEW)
+                && !(old_flags & LIBBALSA_MESSAGE_FLAG_DELETED);
+            is_unread_undeleted =
+                (msg_info->flags & LIBBALSA_MESSAGE_FLAG_NEW)
+                && !(msg_info->flags & LIBBALSA_MESSAGE_FLAG_DELETED);
+            mailbox->unread_messages +=
+                is_unread_undeleted - was_unread_undeleted;
+        }
     }
     libbalsa_unlock_mailbox(mailbox);
 
