@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -44,9 +44,6 @@ struct _LibBalsaMailboxMaildir {
 
     GHashTable* messages_info;
     GPtrArray* msgno_2_msg_info;
-    time_t mtime;
-    time_t mtime_cur;
-    time_t mtime_new;
     gchar *curdir;
     gchar *newdir;
     gchar *tmpdir;
@@ -58,8 +55,6 @@ struct _LibBalsaMailboxMaildirClass {
 
 GType libbalsa_mailbox_maildir_get_type(void);
 GObject *libbalsa_mailbox_maildir_new(const gchar * path, gboolean create);
-gint libbalsa_mailbox_maildir_create(const gchar * path, gboolean create,
-                                     LibBalsaMailboxMaildir * mdir);
 
 #endif
 
