@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -259,15 +259,15 @@ lbm_mbox_stream_seek_to_message(GMimeStream * stream, off_t offset)
         && (nread = g_mime_stream_read(stream, buffer, sizeof buffer))
         == sizeof buffer
         && strncmp("From ", buffer, 5) == 0;
-#if DEBUG_SEEK
     if (!retval) {
         if (nread == sizeof buffer)
             --nread;
         buffer[nread] = 0;
+#if DEBUG_SEEK
         g_print("%s at %ld failed: read %ld chars, saw \"%s\"\n", __func__,
                 (long) offset, (long) nread, buffer);
-    }
 #endif
+    }
 
     g_mime_stream_seek(stream, offset, GMIME_STREAM_SEEK_SET);
 
@@ -490,6 +490,7 @@ parse_mailbox(LibBalsaMailboxMbox * mbox)
 
     libbalsa_mailbox_local_set_threading_info(local);
     msg_info.local_info.message = NULL;
+    msg_info.local_info.loaded  = FALSE;
     while (!g_mime_parser_eos(gmime_parser)) {
 	GMimeMessage *mime_message;
         LibBalsaMessage *msg;
@@ -625,6 +626,7 @@ lbm_mbox_restore(LibBalsaMailboxMbox * mbox)
     end = 0;
     do {
         msg_info->local_info.message = NULL;
+        msg_info->local_info.loaded  = FALSE;
         if (msg_info->start != end)
             /* Error: this message doesn't start at the end of the
              * previous one. */
@@ -1986,6 +1988,7 @@ libbalsa_mailbox_mbox_add_message(LibBalsaMailbox * mailbox,
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_APPEND_ERROR,
                     _("%s: could not open %s."), "MBOX", path);
+        g_free(from);
         return FALSE;
     }
     
