@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -51,10 +51,12 @@ process_mime_part(LibBalsaMessage * message, LibBalsaMessageBody * body,
                   gboolean flow)
 {
     gchar *res = NULL;
+#ifdef HAVE_HTML_WIDGET
+    size_t allocated;
+#endif /* HAVE_HTML_WIDGET */
     GString *reply = NULL;
     gchar *mime_type;
     LibBalsaHTMLType html_type;
-    ssize_t allocated;
 
     switch (libbalsa_message_body_type(body)) {
     case LIBBALSA_MESSAGE_BODY_TYPE_OTHER:
@@ -78,15 +80,19 @@ process_mime_part(LibBalsaMessage * message, LibBalsaMessageBody * body,
 	if (ignore_html && html_type)
 	    break;
 
-
-	allocated = libbalsa_message_body_get_content(body, &res, NULL);
-	if (!res || allocated == -1)
-	    return NULL;
 #ifdef HAVE_HTML_WIDGET
+	allocated = libbalsa_message_body_get_content(body, &res, NULL);
+	if (!res)
+	    return NULL;
+
 	if (html_type) {
 	    allocated = libbalsa_html_filter(html_type, &res, allocated);
 	    libbalsa_html_to_string(&res, allocated);
 	}
+#else  /* HAVE_HTML_WIDGET */
+	libbalsa_message_body_get_content(body, &res, NULL);
+	if (!res)
+	    return NULL;
 #endif /* HAVE_HTML_WIDGET */
 
         if (flow && libbalsa_message_body_is_flowed(body)) {
