@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2003 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,10 +31,6 @@
 #include <gtk/gtk.h>
 #include <gmime/gmime.h>
 
-#if ENABLE_ESMTP
-#include <auth-client.h>
-#endif
-
 #if !USE_GREGEX
 #  ifdef HAVE_PCRE
 #    include <pcreposix.h>
@@ -163,16 +159,15 @@ int libbalsa_safe_rename (const char *src, const char *target);
 gboolean libbalsa_ia_rfc2821_equal(const InternetAddress * a,
 				   const InternetAddress * b);
 
-
-GtkWidget *libbalsa_create_table(guint rows, guint columns);
-GtkWidget *libbalsa_create_label(const gchar * label, GtkWidget * table,
-                                 gint row);
-GtkWidget *libbalsa_create_entry(GtkWidget * table, GCallback func,
-                                 gpointer data, gint row,
-                                 const gchar * initval,
-                                 GtkWidget * hotlabel);
-GtkWidget *libbalsa_create_check(const gchar * label, GtkWidget * table,
-                                 gint row, gboolean initval);
+GtkWidget *libbalsa_create_grid(void);
+GtkWidget *libbalsa_create_grid_label(const gchar * label, GtkWidget * grid,
+                                      gint row);
+GtkWidget *libbalsa_create_grid_entry(GtkWidget * grid, GCallback func,
+                                      gpointer data, gint row,
+                                      const gchar * initval,
+                                      GtkWidget * hotlabel);
+GtkWidget *libbalsa_create_grid_check(const gchar * label, GtkWidget * grid,
+                                      gint row, gboolean initval);
 GtkSizeGroup *libbalsa_create_size_group(GtkWidget * chooser);
 
 void libbalsa_assure_balsa_dir(void);
