@@ -1,7 +1,7 @@
 /* -*-mode:c; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -48,6 +48,8 @@
 #include "information.h"
 
 #if ENABLE_ESMTP
+#include <auth-client.h>
+#include <libesmtp.h>
 #include "smtp-server.h"
 #include <stdarg.h>
 
@@ -907,7 +909,7 @@ lbs_process_queue(LibBalsaMailbox * outbox, LibBalsaFccboxFinder finder,
 
     send_message_info=send_message_info_new(outbox, session, debug);
 
-#ifdef BALSA_USE_THREADS
+#if defined(BALSA_USE_THREADS)
     sending_threads++;
     pthread_create(&send_mail, NULL,
 		   (void *) &balsa_send_message_real, send_message_info);
@@ -1294,7 +1296,7 @@ libbalsa_process_queue(LibBalsaMailbox* outbox, LibBalsaFccboxFinder finder,
 
     send_message_info=send_message_info_new(outbox, finder, debug);
     
-#ifdef BALSA_USE_THREADS
+#if 0 && defined(BALSA_USE_THREADS)
     
     pthread_create(&send_mail, NULL,
 		   (void *) &balsa_send_message_real, send_message_info);
@@ -1391,6 +1393,16 @@ monitor_cb (const char *buf, int buflen, int writing, void *arg)
 
 /* [BCS] radically different since it uses the libESMTP interface.
  */
+
+static gboolean
+balsa_send_message_real_idle_cb(LibBalsaMailbox * outbox)
+{
+    libbalsa_mailbox_close(outbox, TRUE);
+    g_object_unref(outbox);
+
+    return FALSE;
+}
+
 static guint
 balsa_send_message_real(SendMessageInfo* info)
 {
@@ -1455,7 +1467,9 @@ balsa_send_message_real(SendMessageInfo* info)
     smtp_enumerate_messages (info->session, handle_successful_send, 
                              &session_started);
 
-    libbalsa_mailbox_close(info->outbox, TRUE);
+    /* close outbox in an idle callback, as it might affect the display */
+    g_idle_add((GSourceFunc) balsa_send_message_real_idle_cb,
+               g_object_ref(info->outbox));
     /*
      * gdk_flush();
      * gdk_threads_leave();
@@ -1470,6 +1484,8 @@ balsa_send_message_real(SendMessageInfo* info)
         
     smtp_destroy_session (info->session);
     send_message_info_destroy(info);	
+    /* threadmsg is not leaked: */
+    /* cppcheck-suppress memleak */
     return TRUE;
 }
 
@@ -2012,11 +2028,11 @@ static void
 libbalsa_set_message_id(GMimeMessage * mime_message)
 {
     struct utsname utsbuf;
-    gchar *host = "localhost";
+    const gchar *host = "localhost";
     gchar *message_id;
 #ifdef _GNU_SOURCE
     gchar *fqdn;
-    gchar *domain = "localdomain";
+    const gchar *domain = "localdomain";
 
     /* In an ideal world, uname() allows us to make a FQDN. */
     if (uname(&utsbuf) == 0) {
