@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,11 @@
 #include "imap/libimap.h"
 #include "libbalsa.h"
 
-#if defined (HAVE_GNOME_KEYRING)
+#if defined(HAVE_LIBSECRET)
+#include <libsecret/secret.h>
+extern const SecretSchema *LIBBALSA_SERVER_SECRET_SCHEMA;
+#define libbalsa_free_password secret_password_free
+#elif defined (HAVE_GNOME_KEYRING)
 #include <gnome-keyring.h>
 
 #if defined(HAVE_GNOME_KEYRING_24)
@@ -42,7 +46,7 @@ extern const GnomeKeyringPasswordSchema* LIBBALSA_SERVER_KEYRING_SCHEMA;
 #define libbalsa_free_password gnome_keyring_free_password
 #else
 #define libbalsa_free_password g_free
-#endif /* HAVE_GNOME_KEYRING */
+#endif                          /* defined(HAVE_LIBSECRET) */
 
 #define LIBBALSA_TYPE_SERVER \
     (libbalsa_server_get_type())
