@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -35,8 +35,7 @@ extern "C" {
 /*
  * Public methods
  */
-    void balsa_druid_page_finish(GtkAssistant * druid,
-                                 GdkPixbuf * default_logo);
+    void balsa_druid_page_finish(GtkAssistant * druid);
 
 #ifdef __cplusplus
 }
