@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2001 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -120,7 +120,7 @@ balsa_mime_widget_ctx_menu_save(GtkWidget * parent_widget,
     gtk_widget_destroy(save_dialog);
     if (!(save_file = libbalsa_vfs_new_from_uri(file_uri))) {
         balsa_information(LIBBALSA_INFORMATION_ERROR,
-                          _("Could not construct uri from %s"),
+                          _("Could not construct URI from %s"),
                           file_uri);
         g_free(file_uri);
 	return;
@@ -198,43 +198,42 @@ gint
 balsa_mime_widget_key_press_event(GtkWidget * widget, GdkEventKey * event,
 				  BalsaMessage * bm)
 {
-    GtkViewport *viewport;
     GtkAdjustment *adj;
     int page_adjust;
 
-    viewport = GTK_VIEWPORT(bm->cont_viewport);
-    adj = gtk_viewport_get_vadjustment(viewport);
+    adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW
+                                              (bm->scroll));
 
     page_adjust = balsa_app.pgdownmod ?
         (gtk_adjustment_get_page_size(adj) * balsa_app.pgdown_percent) /
         100 : gtk_adjustment_get_page_increment(adj);
 
     switch (event->keyval) {
-    case GDK_Up:
+    case GDK_KEY_Up:
         scroll_change(adj, -gtk_adjustment_get_step_increment(adj), NULL);
         break;
-    case GDK_Down:
+    case GDK_KEY_Down:
         scroll_change(adj, gtk_adjustment_get_step_increment(adj), NULL);
         break;
-    case GDK_Page_Up:
+    case GDK_KEY_Page_Up:
         scroll_change(adj, -page_adjust, NULL);
         break;
-    case GDK_Page_Down:
+    case GDK_KEY_Page_Down:
         scroll_change(adj, page_adjust, NULL);
         break;
-    case GDK_Home:
+    case GDK_KEY_Home:
         if (event->state & GDK_CONTROL_MASK)
             scroll_change(adj, -gtk_adjustment_get_value(adj), NULL);
         else
             return FALSE;
         break;
-    case GDK_End:
+    case GDK_KEY_End:
         if (event->state & GDK_CONTROL_MASK)
             scroll_change(adj, gtk_adjustment_get_upper(adj), NULL);
         else
             return FALSE;
         break;
-    case GDK_F10:
+    case GDK_KEY_F10:
         if (event->state & GDK_SHIFT_MASK) {
 	    GtkWidget *current_widget = balsa_message_current_part_widget(bm);
 
@@ -249,7 +248,7 @@ balsa_mime_widget_key_press_event(GtkWidget * widget, GdkEventKey * event,
         } else
             return FALSE;
         break;
-    case GDK_space:
+    case GDK_KEY_space:
         scroll_change(adj, page_adjust, bm);
         break;
 
