@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2001 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -81,7 +81,7 @@ balsa_mime_widget_signature_widget(LibBalsaMessageBody * mime_body,
 	infostr = labelstr;
     }
     
-    vbox = gtk_vbox_new(FALSE, BMW_VBOX_SPACE);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, BMW_VBOX_SPACE);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), BMW_CONTAINER_BORDER);
     label = gtk_label_new(infostr);
     gtk_label_set_selectable(GTK_LABEL(label), TRUE);
@@ -121,10 +121,10 @@ balsa_mime_widget_crypto_frame(LibBalsaMessageBody * mime_body, GtkWidget * chil
     GtkWidget * vbox;
     GtkWidget * icon_box;
 
-    frame = gtk_frame_new(NULL);       
-    vbox = gtk_vbox_new(FALSE, BMW_VBOX_SPACE);
+    frame = gtk_frame_new(NULL);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, BMW_VBOX_SPACE);
     gtk_container_add(GTK_CONTAINER(frame), vbox);
-    icon_box = gtk_hbox_new(FALSE, BMW_VBOX_SPACE);
+    icon_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, BMW_VBOX_SPACE);
     if (was_encrypted)
 	gtk_box_pack_start(GTK_BOX(icon_box),
 			   gtk_image_new_from_stock(BALSA_PIXMAP_ENCR, GTK_ICON_SIZE_MENU),
@@ -140,10 +140,12 @@ balsa_mime_widget_crypto_frame(LibBalsaMessageBody * mime_body, GtkWidget * chil
     }
     gtk_frame_set_label_widget(GTK_FRAME(frame), icon_box);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), BMW_MESSAGE_PADDING);
+
     gtk_box_pack_start(GTK_BOX(vbox), child, FALSE, FALSE, 0);
 
-    if (signature)
+    if (signature) {
 	gtk_box_pack_end(GTK_BOX(vbox), signature, FALSE, FALSE, 0);
+    }
 
     return frame;
 }
