@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2010 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -37,9 +37,6 @@
 #include "balsa-cite-bar.h"
 
 #if HAVE_GTKSOURCEVIEW
-#ifndef G_CONST_RETURN
-#  define G_CONST_RETURN const
-#endif
 #include <gtksourceview/gtksourceview.h>
 #include <gtksourceview/gtksourcebuffer.h>
 #include <gtksourceview/gtksourcelanguage.h>
@@ -210,7 +207,7 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 	ptr = libbalsa_wrap_rfc2646(ptr, G_MAXINT, FALSE, TRUE, delsp);
     } else if (bm->wrap_text
 #if HAVE_GTKSOURCEVIEW
-	       && !GTK_IS_SOURCE_VIEW(mw->widget)
+	       && !GTK_SOURCE_IS_VIEW(mw->widget)
 #endif
 	       )
 	libbalsa_wrap_string(ptr, balsa_app.browse_wrap_length);
@@ -223,8 +220,6 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 		     (gpointer)mime_body);
 
     buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(mw->widget));
-    allocate_quote_colors(GTK_WIDGET(bm), balsa_app.quoted_color,
-			  0, MAX_QUOTED_COLOR - 1);
 
     url_list = fill_text_buf_cited(mw->widget, ptr,
                                    libbalsa_message_body_is_flowed(mime_body),
@@ -349,7 +344,7 @@ bm_modify_font_from_string(GtkWidget * widget, const char *font)
 {
     PangoFontDescription *desc =
         pango_font_description_from_string(balsa_app.message_font);
-    gtk_widget_modify_font(widget, desc);
+    gtk_widget_override_font(widget, desc);
     pango_font_description_free(desc);
 }
 
@@ -379,11 +374,20 @@ quote_tag(GtkTextBuffer * buffer, gint level, gint margin)
         tag = gtk_text_tag_table_lookup(table, name);
 
         if (!tag) {
+            GdkRGBA *rgba;
+            GdkColor color;
+
+            rgba = &balsa_app.quoted_color[q_level];
+            /* As of now, GtkTextTag has no "foreground-rgba" property :( */
+            color.red   = G_MAXUINT16 * rgba->red;
+            color.green = G_MAXUINT16 * rgba->green;
+            color.blue  = G_MAXUINT16 * rgba->blue;
+            color.pixel = 0;
+
             tag =
-                gtk_text_buffer_create_tag(buffer, name, "foreground-gdk",
-                                           &balsa_app.quoted_color[q_level],
-					   "left-margin",
-                                           BALSA_LEFT_MARGIN
+                gtk_text_buffer_create_tag(buffer, name,
+                                           "foreground-gdk", &color,
+					   "left-margin", BALSA_LEFT_MARGIN
                                            + margin * level,
                                            NULL);
             /* Set a low priority, so we can set both quote color and
@@ -404,15 +408,20 @@ fix_text_widget(GtkWidget *widget, gpointer data)
     GdkWindow *w =
         gtk_text_view_get_window(GTK_TEXT_VIEW(widget),
                                  GTK_TEXT_WINDOW_TEXT);
-    
+
     if (data)
         gdk_window_set_events(w,
                               gdk_window_get_events(w) |
                               GDK_POINTER_MOTION_MASK |
                               GDK_LEAVE_NOTIFY_MASK);
     if (!url_cursor_normal || !url_cursor_over_url) {
-        url_cursor_normal = gdk_cursor_new(GDK_XTERM);
-        url_cursor_over_url = gdk_cursor_new(GDK_HAND2);
+        GdkDisplay *display;
+
+        display = gdk_window_get_display(w);
+        url_cursor_normal =
+            gdk_cursor_new_for_display(display, GDK_XTERM);
+        url_cursor_over_url =
+            gdk_cursor_new_for_display(display, GDK_HAND2);
     }
     gdk_window_set_cursor(w, url_cursor_normal);
 
@@ -422,19 +431,13 @@ fix_text_widget(GtkWidget *widget, gpointer data)
 static void
 gtk_widget_destroy_insensitive(GtkWidget * widget)
 {
-#if GTK_CHECK_VERSION(2, 18, 0)
     if (!gtk_widget_get_sensitive(widget) ||
 	GTK_IS_SEPARATOR_MENU_ITEM(widget))
 	gtk_widget_destroy(widget);
-#else                           /* GTK_CHECK_VERSION(2, 18, 0) */
-    if (!GTK_WIDGET_SENSITIVE(widget) ||
-	GTK_IS_SEPARATOR_MENU_ITEM(widget))
-	gtk_widget_destroy(widget);
-#endif                          /* GTK_CHECK_VERSION(2, 18, 0) */
 }
 
 static void
-structured_phrases_toggle(GtkCheckMenuItem *checkmenuitem, 
+structured_phrases_toggle(GtkCheckMenuItem *checkmenuitem,
 			  GtkTextView *textview)
 {
     GtkTextTagTable * table;
@@ -447,7 +450,7 @@ structured_phrases_toggle(GtkCheckMenuItem *checkmenuitem,
     table = gtk_text_buffer_get_tag_table(gtk_text_view_get_buffer(textview));
     if (!table || phrase_hl == 0 ||
 	(phrase_hl == PHRASE_HIGHLIGHT_ON && new_hl) ||
-	(!phrase_hl == PHRASE_HIGHLIGHT_OFF && !new_hl))
+	(phrase_hl == PHRASE_HIGHLIGHT_OFF && !new_hl))
 	return;
 
     if ((tag = gtk_text_tag_table_lookup(table, "hp-bold")))
@@ -471,8 +474,13 @@ structured_phrases_toggle(GtkCheckMenuItem *checkmenuitem,
 static void
 url_copy_cb(GtkWidget * menu_item, message_url_t * uri)
 {
-    gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY),
-			   uri->url, -1);
+    GdkDisplay *display;
+    GtkClipboard *clipboard;
+
+    display = gtk_widget_get_display(menu_item);
+    clipboard =
+        gtk_clipboard_get_for_display(display, GDK_SELECTION_PRIMARY);
+    gtk_clipboard_set_text(clipboard, uri->url, -1);
 }
 
 static void
@@ -496,17 +504,23 @@ text_view_url_popup(GtkTextView *textview, GtkMenu *menu)
     GList *url_list = g_object_get_data(G_OBJECT(textview), "url-list");
     message_url_t *url;
     gint x, y;
-    GdkModifierType mask;
     GdkWindow *window;
+    GdkDisplay *display;
+    GdkDeviceManager *manager;
+    GdkDevice *device;
     GtkWidget *menu_item;
-    
+
     /* no url list: no check... */
     if (!url_list)
 	return FALSE;
 
     /* check if we are over an url */
     window = gtk_text_view_get_window(textview, GTK_TEXT_WINDOW_TEXT);
-    gdk_window_get_pointer(window, &x, &y, &mask);
+    display = gdk_window_get_display(window);
+    manager = gdk_display_get_device_manager(display);
+    device = gdk_device_manager_get_client_pointer(manager);
+    gdk_window_get_device_position(window, device, &x, &y, NULL);
+
     url = find_url(GTK_WIDGET(textview), x, y, url_list);
     if (!url)
 	return FALSE;
@@ -542,11 +556,9 @@ text_view_populate_popup(GtkTextView *textview, GtkMenu *menu,
     GtkWidget *menu_item;
     gint phrase_hl;
 
-#if GTK_CHECK_VERSION(2, 24, 0)
     gtk_widget_hide(GTK_WIDGET(menu));
-#else                           /* GTK_CHECK_VERSION(2, 24, 0) */
-    gtk_widget_hide_all(GTK_WIDGET(menu));
-#endif                          /* GTK_CHECK_VERSION(2, 24, 0) */
+    gtk_container_foreach(GTK_CONTAINER(menu),
+                          (GtkCallback) gtk_widget_hide, NULL);
     if (text_view_url_popup(textview, menu))
 	return;
 
@@ -588,11 +600,9 @@ store_button_coords(GtkWidget * widget, GdkEventButton * event,
                     gpointer data)
 {
     if (event->type == GDK_BUTTON_PRESS && event->button == 1) {
-        GdkWindow *window =
-            gtk_text_view_get_window(GTK_TEXT_VIEW(widget),
-                                     GTK_TEXT_WINDOW_TEXT);
-
-        gdk_window_get_pointer(window, &stored_x, &stored_y, &stored_mask);
+        stored_x = event->x;
+        stored_y = event->y;
+        stored_mask = event->state;
 
         /* compare only shift, ctrl, and mod1-mod5 */
         stored_mask &= STORED_MASK_BITS;
@@ -615,21 +625,18 @@ check_over_url(GtkWidget * widget, GdkEventMotion * event,
     if (event->type == GDK_LEAVE_NOTIFY)
         url = NULL;
     else {
-	gint x, y;
-	GdkModifierType mask;
-
-        /* FIXME: why can't we just use
-         * x = event->x;
-         * y = event->y;
-         * ??? */
-        gdk_window_get_pointer(window, &x, &y, &mask);
-        url = find_url(widget, x, y, url_list);
+        url = find_url(widget, (gint) event->x, (gint) event->y, url_list);
     }
 
     if (url) {
         if (!url_cursor_normal || !url_cursor_over_url) {
-            url_cursor_normal = gdk_cursor_new(GDK_LEFT_PTR);
-            url_cursor_over_url = gdk_cursor_new(GDK_HAND2);
+            GdkDisplay *display;
+
+            display = gdk_window_get_display(window);
+            url_cursor_normal =
+                gdk_cursor_new_for_display(display, GDK_LEFT_PTR);
+            url_cursor_over_url =
+                gdk_cursor_new_for_display(display, GDK_HAND2);
         }
         if (!was_over_url) {
             gdk_window_set_cursor(window, url_cursor_over_url);
@@ -796,6 +803,7 @@ handle_url(const gchar * url)
         GtkStatusbar *statusbar;
         guint context_id;
         gchar *notice = g_strdup_printf(_("Calling URL %s..."), url);
+        GdkScreen *screen;
         GError *err = NULL;
 
         statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
@@ -805,7 +813,8 @@ handle_url(const gchar * url)
         gtk_statusbar_push(statusbar, context_id, notice);
         SCHEDULE_BAR_REFRESH();
         g_free(notice);
-        gtk_show_uri(NULL, url, gtk_get_current_event_time(), &err);
+        screen = gtk_widget_get_screen(GTK_WIDGET(balsa_app.main_window));
+        gtk_show_uri(screen, url, gtk_get_current_event_time(), &err);
         if (err) {
             balsa_information(LIBBALSA_INFORMATION_WARNING,
                               _("Error showing %s: %s\n"),
@@ -926,39 +935,44 @@ draw_cite_bar_real(cite_bar_t * bar, cite_bar_draw_mode_t * draw_mode)
 
     /* initialise iters if we don't have the widget yet */
     if (!bar->bar) {
-	gtk_text_buffer_get_iter_at_offset(draw_mode->buffer, &bar->start_iter,
-					   bar->start_offs);
-	gtk_text_buffer_get_iter_at_offset(draw_mode->buffer, &bar->end_iter,
-					   bar->end_offs);
+        gtk_text_buffer_get_iter_at_offset(draw_mode->buffer,
+                                           &bar->start_iter,
+                                           bar->start_offs);
+        gtk_text_buffer_get_iter_at_offset(draw_mode->buffer,
+                                           &bar->end_iter,
+                                           bar->end_offs);
     }
 
     /* get the locations */
-    gtk_text_view_get_iter_location(draw_mode->view, &bar->start_iter, &location);
-    gtk_text_view_buffer_to_window_coords(draw_mode->view, GTK_TEXT_WINDOW_TEXT,
-					  location.x, location.y,
-					  &x_pos, &y_pos);
-    gtk_text_view_get_iter_location(draw_mode->view, &bar->end_iter, &location);
-    gtk_text_view_buffer_to_window_coords(draw_mode->view, GTK_TEXT_WINDOW_TEXT,
-					  location.x, location.y,
-					  &x_pos, &height);
+    gtk_text_view_get_iter_location(draw_mode->view, &bar->start_iter,
+                                    &location);
+    gtk_text_view_buffer_to_window_coords(draw_mode->view,
+                                          GTK_TEXT_WINDOW_TEXT, location.x,
+                                          location.y, &x_pos, &y_pos);
+    gtk_text_view_get_iter_location(draw_mode->view, &bar->end_iter,
+                                    &location);
+    gtk_text_view_buffer_to_window_coords(draw_mode->view,
+                                          GTK_TEXT_WINDOW_TEXT, location.x,
+                                          location.y, &x_pos, &height);
     height -= y_pos;
 
     /* add a new widget if necessary */
-    if (bar->bar == NULL) {
-	bar->bar = balsa_cite_bar_new(height, bar->depth, draw_mode->dimension);
-	gtk_widget_modify_fg(bar->bar, GTK_STATE_NORMAL,
-			     &balsa_app.quoted_color[(bar->depth - 1) % MAX_QUOTED_COLOR]);
-        gtk_widget_modify_bg(bar->bar, GTK_STATE_NORMAL,
-                             &gtk_widget_get_style(GTK_WIDGET
-                                                   (draw_mode->view))->
-                             base[GTK_STATE_NORMAL]);
-	gtk_widget_show(bar->bar);
-	gtk_text_view_add_child_in_window(draw_mode->view, bar->bar,
-					  GTK_TEXT_WINDOW_TEXT, 0, y_pos);
+    if (!bar->bar) {
+        bar->bar =
+            balsa_cite_bar_new(height, bar->depth, draw_mode->dimension);
+
+        gtk_widget_override_color(bar->bar, GTK_STATE_FLAG_NORMAL,
+                                  &balsa_app.
+                                  quoted_color[(bar->depth -
+                                                1) % MAX_QUOTED_COLOR]);
+
+        gtk_widget_show(bar->bar);
+        gtk_text_view_add_child_in_window(draw_mode->view, bar->bar,
+                                          GTK_TEXT_WINDOW_TEXT, 0, y_pos);
     } else if (bar->y_pos != y_pos || bar->height != height) {
-	/* shift/resize existing widget */
-	balsa_cite_bar_resize(BALSA_CITE_BAR(bar->bar), height);
-	gtk_text_view_move_child(draw_mode->view, bar->bar, 0, y_pos);
+        /* shift/resize existing widget */
+        balsa_cite_bar_resize(BALSA_CITE_BAR(bar->bar), height);
+        gtk_text_view_move_child(draw_mode->view, bar->bar, 0, y_pos);
     }
 
     /* remember current values */
@@ -1017,6 +1031,12 @@ bm_zoom_reset(BalsaMessage * bm)
     balsa_message_zoom(bm, 0);
 }
 
+static void
+bm_select_all_cb(GtkWidget * html)
+{
+    libbalsa_html_select_all(html);
+}
+
 static void
 bmwt_populate_popup_menu(BalsaMessage * bm,
                          GtkWidget    * html,
@@ -1045,6 +1065,16 @@ bmwt_populate_popup_menu(BalsaMessage * bm,
     menuitem = gtk_separator_menu_item_new();
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
+    if (libbalsa_html_can_select(html)) {
+        menuitem = gtk_menu_item_new_with_mnemonic(_("Select _All"));
+        g_signal_connect_swapped(G_OBJECT(menuitem), "activate",
+                                 G_CALLBACK(bm_select_all_cb), html);
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+
+        menuitem = gtk_separator_menu_item_new();
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+    }
+
     libbalsa_vfs_fill_menu_by_content_type(GTK_MENU(menu), "text/html",
                                            G_CALLBACK
                                            (balsa_mime_widget_ctx_menu_cb),
@@ -1071,15 +1101,29 @@ static gboolean
 balsa_gtk_html_popup(GtkWidget * html, BalsaMessage * bm)
 {
     GtkWidget *menu;
+    const GdkEvent *event;
+    GdkEvent *current_event = NULL;
+    guint32 time;
+    guint button;
 
     menu = gtk_menu_new();
     bmwt_populate_popup_menu(bm, html, GTK_MENU(menu));
 
     gtk_widget_show_all(menu);
-    g_object_ref_sink(menu);
-    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
-                   0, gtk_get_current_event_time());
-    g_object_unref(menu);
+
+    /* In WebKit2, the context menu signal is asynchronous, so the
+     * GdkEvent is no longer current; instead it is preserved and passed
+     * to us: */
+    event = g_object_get_data(G_OBJECT(html), LIBBALSA_HTML_POPUP_EVENT);
+    if (!event)
+        event = current_event = gtk_get_current_event();
+    time = gdk_event_get_time(event);
+    button = 0;
+    gdk_event_get_button(event, &button);
+    if (current_event)
+        gdk_event_free(current_event);
+
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, time);
 
     return TRUE;
 }
@@ -1106,7 +1150,7 @@ bmwt_populate_popup_cb(GtkWidget * widget, GtkMenu * menu, gpointer data)
     gtk_widget_show_all(GTK_WIDGET(menu));
 }
 
-BalsaMimeWidget *
+static BalsaMimeWidget *
 bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body)
 {
     BalsaMimeWidget *mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
@@ -1136,14 +1180,15 @@ bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body)
 }
 #endif /* defined HAVE_HTML_WIDGET */
 
-#define TABLE_ATTACH(t,str,label) \
-    if(str) { GtkWidget *lbl = gtk_label_new(label);              \
-        gtk_table_attach(t, lbl, 0, 1, row, row+1,                \
-                         GTK_FILL, GTK_FILL, 4, 2);               \
-        gtk_misc_set_alignment(GTK_MISC(lbl), 1.0, 0.0); \
-        gtk_table_attach(table, lbl=gtk_label_new(str), 1, 2, row, row+1, \
-                         GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 4, 2); \
-        gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.0); \
+#define GRID_ATTACH(g,str,label)                                  \
+    if(str) { GtkWidget *lbl;                                     \
+        lbl = gtk_label_new(label);                               \
+        gtk_widget_set_halign(lbl, GTK_ALIGN_START);                \
+        gtk_grid_attach(g, lbl, 0, row, 1, 1);                    \
+        lbl = gtk_label_new(str);                                 \
+        gtk_label_set_line_wrap(GTK_LABEL(lbl), TRUE);            \
+        gtk_widget_set_halign(lbl, GTK_ALIGN_START);              \
+        gtk_grid_attach(g, lbl, 1, row, 1, 1);                    \
         row++;                                                    \
     }
 
@@ -1152,40 +1197,44 @@ bm_widget_new_vcard(BalsaMessage *bm, LibBalsaMessageBody *mime_body,
                     gchar *ptr, size_t len)
 {
     BalsaMimeWidget *mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
-    LibBalsaAddress * addr =
-	libbalsa_address_new_from_vcard(ptr, mime_body->charset ? mime_body->charset : "us-ascii");
-    GtkTable *table;
+    LibBalsaAddress *addr;
+    GtkGrid *grid;
     GtkWidget *w;
     int row = 1;
 
+    addr =
+        libbalsa_address_new_from_vcard(ptr, mime_body->charset ?
+                                        mime_body-> charset : "us-ascii");
     if (!addr)
         return NULL;
 
-    mw->widget = gtk_table_new(10, 2, FALSE);
-    table = (GtkTable*)mw->widget;
-        
-    gtk_table_attach_defaults(table, w=gtk_label_new(_("Address")),
-                              0, 1, 0, 1);
-    gtk_misc_set_alignment(GTK_MISC(w), 1.0, 0.0);
-    w = gtk_button_new_with_mnemonic(_("S_tore"));
-    /* FIXME: Connect signal */
-    gtk_table_attach(table, w, 1, 2, 0, 1, 0,0,  2, 2);
+    mw->widget = gtk_grid_new();
+    grid = (GtkGrid*)mw->widget;
+#if GTK_CHECK_VERSION(3, 2, 0)
+    gtk_grid_set_row_spacing(grid, 6);
+    gtk_grid_set_column_spacing(grid, 12);
+#else                           /* GTK_CHECK_VERSION(3, 2, 0) */
+    gtk_grid_set_row_spacing(grid, 12);
+    gtk_grid_set_column_spacing(grid, 6);
+#endif                          /* GTK_CHECK_VERSION(3, 2, 0) */
+
+    w = gtk_button_new_with_mnemonic(_("S_tore Address"));
+    gtk_grid_attach(grid, w, 0, 0, 2, 1);
     g_signal_connect_swapped(w, "clicked",
                              G_CALLBACK(balsa_store_address), addr);
     g_object_weak_ref(G_OBJECT(mw), (GWeakNotify)g_object_unref, addr);
- 
 
-    TABLE_ATTACH(table, addr->full_name,    _("Full Name"));
-    TABLE_ATTACH(table, addr->nick_name,    _("Nick Name"));
-    TABLE_ATTACH(table, addr->first_name,   _("First Name"));
-    TABLE_ATTACH(table, addr->last_name,    _("Last Name"));
-    TABLE_ATTACH(table, addr->organization, _("Organization"));
+
+    GRID_ATTACH(grid, addr->full_name,    _("Full Name:"));
+    GRID_ATTACH(grid, addr->nick_name,    _("Nick Name:"));
+    GRID_ATTACH(grid, addr->first_name,   _("First Name:"));
+    GRID_ATTACH(grid, addr->last_name,    _("Last Name:"));
+    GRID_ATTACH(grid, addr->organization, _("Organization:"));
     if(addr->address_list) {
-        TABLE_ATTACH(table, addr->address_list->data, _("Email Address"));
+        GRID_ATTACH(grid, addr->address_list->data, _("Email Address:"));
     }
-        
+
     g_object_set_data(G_OBJECT(mw->widget), "mime-body", mime_body);
-    gtk_widget_show_all(mw->widget);
     return mw;
 }
 
@@ -1234,6 +1283,7 @@ fill_text_buf_cited(GtkWidget *widget, const gchar *text_body,
     LibBalsaUrlInsertInfo url_info;
     GList * cite_bars_list;
     guint cite_level;
+    GdkScreen *screen;
     guint cite_start;
     gint margin;
     gdouble char_width;
@@ -1248,6 +1298,7 @@ fill_text_buf_cited(GtkWidget *widget, const gchar *text_body,
     regex_t rex;
 #endif                          /* USE_GREGEX */
     gboolean have_regex;
+    GdkColor color;
 
     /* prepare citation regular expression for plain bodies */
     if (is_plain) {
@@ -1270,12 +1321,15 @@ fill_text_buf_cited(GtkWidget *widget, const gchar *text_body,
         char_width = char_width / PANGO_SCALE;
 
     /* convert char_width from points to pixels */
-    margin = (char_width / 72.0) *
-        gdk_screen_get_resolution(gdk_screen_get_default());
+    screen = gtk_widget_get_screen(widget);
+    margin = (char_width / 72.0) * gdk_screen_get_resolution(screen);
 
+    color.red   = G_MAXUINT16 * balsa_app.url_color.red;
+    color.green = G_MAXUINT16 * balsa_app.url_color.green;
+    color.blue  = G_MAXUINT16 * balsa_app.url_color.blue;
+    color.pixel = 0;
     gtk_text_buffer_create_tag(buffer, "url",
-                               "foreground-gdk",
-                               &balsa_app.url_color, NULL);
+                               "foreground-gdk", &color, NULL);
     gtk_text_buffer_create_tag(buffer, "emphasize",
                                "foreground", "red",
                                "underline", PANGO_UNDERLINE_SINGLE,
@@ -1379,7 +1433,7 @@ fill_text_buf_cited(GtkWidget *widget, const gchar *text_body,
         g_object_set_data_full(G_OBJECT(widget), "cite-bars", cite_bars_list,
                                (GDestroyNotify) destroy_cite_bars);
         g_object_set_data(G_OBJECT(widget), "cite-margin", GINT_TO_POINTER(margin));
-        g_signal_connect_after(G_OBJECT(widget), "expose-event",
+        g_signal_connect_after(G_OBJECT(widget), "draw",
                                G_CALLBACK(draw_cite_bars), cite_bars_list);
     }
 
