@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2001 Stuart Parmenter and others
+ * Copyright (C) 1997-2013 Stuart Parmenter and others
  * Written by (C) Albrecht Dre <albrecht.dress@arcor.de> 2007
  *
  * This program is free software; you can redistribute it and/or modify
@@ -487,8 +487,9 @@ balsa_print_object_text_vcard(GList * list,
 				 GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
     if (!pod->pixbuf) {
 	gchar *conttype = libbalsa_message_body_get_mime_type(body);
-	
-	pod->pixbuf = libbalsa_icon_finder(conttype, NULL, NULL, GTK_ICON_SIZE_DND);
+
+	pod->pixbuf = libbalsa_icon_finder(NULL, conttype, NULL, NULL,
+                                           GTK_ICON_SIZE_DND);
     }
     pod->c_image_width = gdk_pixbuf_get_width(pod->pixbuf);
     pod->c_image_height = gdk_pixbuf_get_height(pod->pixbuf);
@@ -633,8 +634,9 @@ balsa_print_object_text_calendar(GList * list,
 				 GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
     if (!pod->pixbuf) {
 	gchar *conttype = libbalsa_message_body_get_mime_type(body);
-	
-	pod->pixbuf = libbalsa_icon_finder(conttype, NULL, NULL, GTK_ICON_SIZE_DND);
+
+	pod->pixbuf = libbalsa_icon_finder(NULL, conttype, NULL, NULL,
+                                           GTK_ICON_SIZE_DND);
     }
     pod->c_image_width = gdk_pixbuf_get_width(pod->pixbuf);
     pod->c_image_height = gdk_pixbuf_get_height(pod->pixbuf);
@@ -803,9 +805,9 @@ balsa_print_object_text_draw(BalsaPrintObject * self,
 	    gint k = (po->cite_level - 1) % MAX_QUOTED_COLOR;
 
 	    cairo_set_source_rgb(cairo_ctx,
-				 (gdouble) balsa_app.quoted_color[k].red / 65536.0,
-				 (gdouble) balsa_app.quoted_color[k].green / 65536.0,
-				 (gdouble) balsa_app.quoted_color[k].blue / 65536.0);
+				 balsa_app.quoted_color[k].red,
+				 balsa_app.quoted_color[k].green,
+				 balsa_app.quoted_color[k].blue);
 	}
     }
     cairo_move_to(cairo_ctx, self->c_at_x + po->cite_level * C_LABEL_SEP,
