@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -285,12 +285,12 @@ void balsa_filter_run_dialog_init(BalsaFilterRunDialog * p)
     g_signal_connect(G_OBJECT(p), "destroy",
                      G_CALLBACK(fr_destroy_window_cb), NULL);
 
-    hbox = gtk_hbox_new(FALSE,2);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL,2);
 
     gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(p))),
 		       hbox, TRUE, TRUE, 0);
 
-    vbox = gtk_vbox_new(FALSE, 2);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 
     p->available_filters =
@@ -308,7 +308,7 @@ void balsa_filter_run_dialog_init(BalsaFilterRunDialog * p)
     gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
 
     /* To keep a consistent look, make a button box for a single button. */
-    bbox = gtk_hbutton_box_new();
+    bbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
     gtk_box_set_spacing(GTK_BOX(bbox), 2);
     gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_SPREAD);
     gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 2);
@@ -319,9 +319,9 @@ void balsa_filter_run_dialog_init(BalsaFilterRunDialog * p)
     g_signal_connect_swapped(G_OBJECT(button), "clicked",
                              G_CALLBACK(fr_apply_selected_pressed), p);
     gtk_container_add(GTK_CONTAINER(bbox), button);
- 
+
     /* Buttons between the 2 lists */
-    bbox = gtk_vbutton_box_new();
+    bbox = gtk_button_box_new(GTK_ORIENTATION_VERTICAL);
     gtk_box_set_spacing(GTK_BOX(bbox), 2);
     gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_SPREAD);
 
@@ -338,7 +338,7 @@ void balsa_filter_run_dialog_init(BalsaFilterRunDialog * p)
 
     gtk_box_pack_start(GTK_BOX(hbox),bbox, FALSE, FALSE, 6);
 
-    vbox=gtk_vbox_new(FALSE,2);
+    vbox=gtk_box_new(GTK_ORIENTATION_VERTICAL,2);
 
     gtk_box_pack_start(GTK_BOX(hbox),vbox, TRUE, TRUE, 0);
 
@@ -356,7 +356,7 @@ void balsa_filter_run_dialog_init(BalsaFilterRunDialog * p)
     gtk_box_pack_start(GTK_BOX(vbox),sw, TRUE, TRUE, 0);
 
     /* up down arrow buttons */
-    bbox = gtk_hbutton_box_new();
+    bbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
     gtk_box_set_spacing(GTK_BOX(bbox), 2);
     gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_SPREAD);
 
