@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -43,7 +43,7 @@ void mailbox_conf_edit_cb(GtkWidget * widget, gpointer data);
 /* Helpers for dialogs. */
 BalsaMailboxConfView *mailbox_conf_view_new(LibBalsaMailbox * mailbox,
                                             GtkWindow * window,
-                                            GtkWidget * table, gint row,
+                                            GtkWidget * grid, gint row,
                                             GCallback callback);
 void mailbox_conf_view_check(BalsaMailboxConfView * mcc,
                              LibBalsaMailbox * mailbox);
@@ -52,7 +52,7 @@ void mailbox_conf_view_check(BalsaMailboxConfView * mcc,
 typedef struct {
     GtkWidget *use_ssl;
     GtkWidget *tls_mode;
-    GtkTable  *table;      /* internal */
+    GtkGrid   *grid;       /* internal */
     GtkWidget *tls_option; /* internal */
     GtkWidget *server;     /* internal */
     const gchar *default_ports;
