@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2010 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
- * 02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __MAIN_WINDOW_H__
@@ -30,7 +28,9 @@
 #include <libnotify/notify.h>
 #endif
 
-#if defined(HAVE_LIBNM_GLIB)
+#if GLIB_CHECK_VERSION(2, 32, 0)
+#include <gio/gio.h>
+#elif defined(HAVE_LIBNM_GLIB)
 #include <nm-client.h>
 #endif
 
@@ -42,6 +42,9 @@
 #define BALSA_WINDOW_CLASS(klass)	       (G_TYPE_CHECK_CLASS_CAST (klass, BALSA_TYPE_WINDOW, BalsaWindowClass))
 #define BALSA_IS_WINDOW(obj)		       (G_TYPE_CHECK_INSTANCE_TYPE (obj, BALSA_TYPE_WINDOW))
 #define BALSA_IS_WINDOW_CLASS(klass)	       (G_TYPE_CHECK_CLASS_TYPE (klass, BALSA_TYPE_WINDOW))
+#define BALSA_WINDOW_GET_CLASS(window)                       \
+    (G_TYPE_INSTANCE_GET_CLASS ((window), BALSA_TYPE_WINDOW, \
+				BalsaWindowClass))
 
 /* Type values for mailbox checking */
 enum MailboxCheckType {
@@ -63,6 +66,7 @@ struct _BalsaWindow {
 
     GtkWidget *toolbar;
     GtkWidget *sos_bar;
+    GtkWidget *bottom_bar;
     GtkWidget *progress_bar;
     GtkWidget *statusbar;
     GtkWidget *mblist;
@@ -96,10 +100,14 @@ struct _BalsaWindow {
     NotifyNotification *new_mail_note;
 #endif                         /* HAVE_NOTIFY */
 
-#if defined(HAVE_LIBNM_GLIB)
+#if GLIB_CHECK_VERSION(2, 32, 0)
+    /* Support GNetworkMonitor: */
+    gboolean network_available;
+    time_t last_check_time;
+#elif defined(HAVE_LIBNM_GLIB)
     /* NetworkManager state */
     NMState nm_state;
-    gboolean check_mail_skipped;
+    time_t last_check_time;
 #endif                          /* defined(HAVE_LIBNM_GLIB) */
 };
 
