@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2010 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -552,7 +552,7 @@ message_window_new(LibBalsaMailbox * mailbox, guint msgno)
     mw = g_malloc0(sizeof(MessageWindow));
 
     mw->window = window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    vbox = gtk_vbox_new(FALSE, 0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_container_add(GTK_CONTAINER(window), vbox);
 
     mw->headers_shown=balsa_app.shown_headers;
@@ -652,7 +652,7 @@ message_window_new(LibBalsaMailbox * mailbox, guint msgno)
     close_widget =
         gtk_ui_manager_get_widget(ui_manager, "/MainMenu/FileMenu/Close");
     gtk_widget_add_accelerator(close_widget, "activate", accel_group,
-                               GDK_Escape, (GdkModifierType) 0,
+                               GDK_KEY_Escape, (GdkModifierType) 0,
                                (GtkAccelFlags) 0);
 }
 
