@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2010 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,9 +26,8 @@
 
 #include <stdlib.h>
 #include <string.h>
-#if HAVE_GNOME
-#include <gconf/gconf-client.h>
-#endif
+#include <gio/gio.h>
+#include <gio/gdesktopappinfo.h>
 #include <glib/gi18n.h>
 #include "balsa-app.h"
 #include "server.h"
@@ -54,7 +53,6 @@
 #define ADDRESS_BOOK_SECTION_PREFIX "address-book-"
 #define IDENTITY_SECTION_PREFIX "identity-"
 #define VIEW_SECTION_PREFIX "view-"
-#define VIEW_BY_URL_SECTION_PREFIX "viewByUrl-"
 #define SMTP_SERVER_SECTION_PREFIX "smtp-server-"
 
 static gint config_global_load(void);
@@ -62,16 +60,16 @@ static gint config_folder_init(const gchar * group);
 static gint config_mailbox_init(const gchar * group);
 static gchar *config_get_unused_group(const gchar * group);
 
-static void save_color(gchar * key, GdkColor * color);
-static void load_color(gchar * key, GdkColor * color);
+static void save_color(const gchar * key, GdkRGBA * rgba);
+static void load_color(const gchar * name,
+                       const gchar * def_val,
+                       GdkRGBA     * rgba);
 static void save_mru(GList  *mru, const gchar * group);
 static void load_mru(GList **mru, const gchar * group);
 
 static void config_address_books_save(void);
 static void config_identities_load(void);
 
-static void check_for_old_sigs(GList * id_list_tmp);
-
 static void config_filters_load(void);
 
 #define folder_section_path(mn) \
@@ -113,6 +111,7 @@ config_load_sections(void)
     libbalsa_conf_foreach_group(MAILBOX_SECTION_PREFIX,
                                 config_load_section,
                                 config_mailbox_init);
+    balsa_app.inbox_input = g_list_reverse(balsa_app.inbox_input);
     libbalsa_conf_foreach_group(FOLDER_SECTION_PREFIX,
                                 config_load_section,
                                 config_folder_init);
@@ -261,7 +260,7 @@ config_address_book_save(LibBalsaAddressBook * ab)
 
     g_free(group);
 
-    libbalsa_conf_sync();
+    libbalsa_conf_queue_sync();
 }
 
 void
@@ -270,7 +269,7 @@ config_address_book_delete(LibBalsaAddressBook * ab)
     if (ab->config_prefix) {
 	libbalsa_conf_remove_group(ab->config_prefix);
 	libbalsa_conf_private_remove_group(ab->config_prefix);
-	libbalsa_conf_sync();
+	libbalsa_conf_queue_sync();
     }
 }
 
@@ -294,7 +293,7 @@ gint config_mailbox_add(LibBalsaMailbox * mailbox, const char *key_arg)
     libbalsa_conf_pop_group();
     g_free(tmp);
 
-    libbalsa_conf_sync();
+    libbalsa_conf_queue_sync();
     return TRUE;
 }				/* config_mailbox_add */
 
@@ -317,7 +316,7 @@ gint config_folder_add(BalsaMailboxNode * mbnode, const char *key_arg)
     libbalsa_conf_pop_group();
     g_free(tmp);
 
-    libbalsa_conf_sync();
+    libbalsa_conf_queue_sync();
     return TRUE;
 }				/* config_mailbox_add */
 
@@ -329,7 +328,7 @@ gint config_mailbox_delete(const LibBalsaMailbox * mailbox)
     tmp = mailbox_section_path(mailbox);
     res = libbalsa_conf_has_group(tmp);
     libbalsa_conf_remove_group(tmp);
-    libbalsa_conf_sync();
+    libbalsa_conf_queue_sync();
     g_free(tmp);
     return res;
 }				/* config_mailbox_delete */
@@ -342,7 +341,7 @@ config_folder_delete(const BalsaMailboxNode * mbnode)
     tmp = folder_section_path(mbnode);
     res = libbalsa_conf_has_group(tmp);
     libbalsa_conf_remove_group(tmp);
-    libbalsa_conf_sync();
+    libbalsa_conf_queue_sync();
     g_free(tmp);
     return res;
 }				/* config_folder_delete */
@@ -361,7 +360,7 @@ gint config_mailbox_update(LibBalsaMailbox * mailbox)
     libbalsa_mailbox_save_config(mailbox, key);
     g_free(key);
     libbalsa_conf_pop_group();
-    libbalsa_conf_sync();
+    libbalsa_conf_queue_sync();
     return res;
 }				/* config_mailbox_update */
 
@@ -379,7 +378,7 @@ gint config_folder_update(BalsaMailboxNode * mbnode)
     balsa_mailbox_node_save_config(mbnode, key);
     g_free(key);
     libbalsa_conf_pop_group();
-    libbalsa_conf_sync();
+    libbalsa_conf_queue_sync();
     return res;
 }				/* config_folder_update */
 
@@ -409,6 +408,7 @@ pop3_progress_notify(LibBalsaMailbox* mailbox, int msg_type, int prog, int tot,
     while(gtk_events_pending())
         gtk_main_iteration_do(FALSE);
 #endif
+    /* cppcheck-suppress memleak */
 }
 
 /* Initialize the specified mailbox, creating the internal data
@@ -438,8 +438,8 @@ config_mailbox_init(const gchar * prefix)
                          "progress-notify", G_CALLBACK(pop3_progress_notify),
                          mailbox);
 	balsa_app.inbox_input =
-	    g_list_append(balsa_app.inbox_input, 
-			  balsa_mailbox_node_new_from_mailbox(mailbox));
+            g_list_prepend(balsa_app.inbox_input,
+                           balsa_mailbox_node_new_from_mailbox(mailbox));
     } else {
         LibBalsaMailbox **special = NULL;
 	BalsaMailboxNode *mbnode;
@@ -614,9 +614,6 @@ config_global_load(void)
  			     filter_strerror(filter_errno));
     }
 
-    /* find and convert old-style signature entries */
-    check_for_old_sigs(balsa_app.identities);
-
     /* Section for the balsa_information() settings... */
     libbalsa_conf_push_group("InformationMessages");
 
@@ -740,19 +737,19 @@ config_global_load(void)
 	int i;
 	gchar *default_quoted_color[MAX_QUOTED_COLOR] = {"#088", "#800"};
 	for(i=0;i<MAX_QUOTED_COLOR;i++) {
-	    gchar *text = g_strdup_printf("QuotedColor%d=%s", i, i<6 ?
-			  default_quoted_color[i] : DEFAULT_QUOTED_COLOR);
-	    load_color(text, &balsa_app.quoted_color[i]);
-	    g_free(text);
+            gchar *key;
+            const gchar *def_val;
+
+            key = g_strdup_printf("QuotedColor%d", i);
+            def_val =
+                i < 6 ? default_quoted_color[i] : DEFAULT_QUOTED_COLOR;
+            load_color(key, def_val, &balsa_app.quoted_color[i]);
+            g_free(key);
 	}
     }
 
     /* URL coloring */
-    load_color("UrlColor=" DEFAULT_URL_COLOR, &balsa_app.url_color);
-
-    /* bad address coloring */
-    load_color("BadAddressColor=" DEFAULT_BAD_ADDRESS_COLOR,
-               &balsa_app.bad_address_color);
+    load_color("UrlColor", DEFAULT_URL_COLOR, &balsa_app.url_color);
 
     /* ... font used to display messages */
     g_free(balsa_app.message_font);
@@ -851,6 +848,12 @@ config_global_load(void)
     balsa_app.source_height = libbalsa_conf_get_int("Height=400");
     libbalsa_conf_pop_group();
 
+    /* MRU mailbox tree ... */
+    libbalsa_conf_push_group("MruTree");
+    balsa_app.mru_tree_width  = libbalsa_conf_get_int("Width=150");
+    balsa_app.mru_tree_height = libbalsa_conf_get_int("Height=300");
+    libbalsa_conf_pop_group();
+
     /* Printing options ... */
     libbalsa_conf_push_group("Printing");
 
@@ -898,11 +901,6 @@ config_global_load(void)
     if (def_used)
         balsa_app.spell_check_active = balsa_app.spell_check_lang != NULL;
 #else                           /* HAVE_GTKSPELL */
-    balsa_app.module = d_get_gint("PspellModule", DEFAULT_PSPELL_MODULE);
-    balsa_app.suggestion_mode =
-	d_get_gint("PspellSuggestMode", DEFAULT_PSPELL_SUGGEST_MODE);
-    balsa_app.ignore_size =
-	d_get_gint("PspellIgnoreSize", DEFAULT_PSPELL_IGNORE_SIZE);
     balsa_app.check_sig =
 	d_get_gint("SpellCheckSignature", DEFAULT_CHECK_SIG);
     balsa_app.check_quoted =
@@ -1287,7 +1285,6 @@ config_save(void)
     }
 
     save_color("UrlColor", &balsa_app.url_color);
-    save_color("BadAddressColor", &balsa_app.bad_address_color);
 
     /* ... handling of Multipart/Alternative */
     libbalsa_conf_set_bool("DisplayAlternativeAsPlain",
@@ -1346,6 +1343,12 @@ config_save(void)
     libbalsa_conf_set_int("Height", balsa_app.source_height);
     libbalsa_conf_pop_group();
 
+    /* MRU mailbox tree ... */
+    libbalsa_conf_push_group("MruTree");
+    libbalsa_conf_set_int("Width",  balsa_app.mru_tree_width);
+    libbalsa_conf_set_int("Height", balsa_app.mru_tree_height);
+    libbalsa_conf_pop_group();
+
     /* Printing options ... */
     libbalsa_conf_push_group("Printing");
     save_gtk_page_setup(balsa_app.page_setup);
@@ -1383,9 +1386,6 @@ config_save(void)
     libbalsa_conf_set_bool("SpellCheckActive", 
                            balsa_app.spell_check_active);
 #else                           /* HAVE_GTKSPELL */
-    libbalsa_conf_set_int("PspellModule", balsa_app.module);
-    libbalsa_conf_set_int("PspellSuggestMode", balsa_app.suggestion_mode);
-    libbalsa_conf_set_int("PspellIgnoreSize", balsa_app.ignore_size);
     libbalsa_conf_set_int("SpellCheckSignature", balsa_app.check_sig);
     libbalsa_conf_set_int("SpellCheckQuoted", balsa_app.check_quoted);
 #endif                          /* HAVE_GTKSPELL */
@@ -1583,7 +1583,7 @@ config_address_book_load(const gchar * key, const gchar * value,
 
     if (address_book) {
         balsa_app.address_book_list =
-            g_list_append(balsa_app.address_book_list, address_book);
+            g_list_prepend(balsa_app.address_book_list, address_book);
 
         if (default_address_book_prefix
             && strcmp(key, default_address_book_prefix) == 0) {
@@ -1605,15 +1605,16 @@ config_address_books_load(void)
     libbalsa_conf_pop_group();
 
     /* Free old data in case address books were set by eg. config druid. */
-    g_list_foreach(balsa_app.address_book_list, (GFunc) g_object_unref,
-                   NULL);
-    g_list_free(balsa_app.address_book_list);
+    g_list_free_full(balsa_app.address_book_list, g_object_unref);
     balsa_app.address_book_list = NULL;
 
     libbalsa_conf_foreach_group(ADDRESS_BOOK_SECTION_PREFIX,
                                 config_address_book_load,
                                 default_address_book_prefix);
 
+    balsa_app.address_book_list =
+        g_list_reverse(balsa_app.address_book_list);
+
     g_free(default_address_book_prefix);
 }
 
@@ -1680,8 +1681,7 @@ config_identities_load()
     gchar *default_ident;
 
     /* Free old data in case identities were set by eg. config druid. */
-    g_list_foreach(balsa_app.identities, (GFunc) g_object_unref, NULL);
-    g_list_free(balsa_app.identities);
+    g_list_free_full(balsa_app.identities, g_object_unref);
     balsa_app.identities = NULL;
 
     libbalsa_conf_push_group("identity");
@@ -1737,107 +1737,6 @@ config_identities_save(void)
     }
 }
 
-static gboolean
-config_view_load(const gchar * key, const gchar * value, gpointer data)
-{
-    gboolean compat = GPOINTER_TO_INT(data);
-    gchar *url;
-    gint def;
-
-    libbalsa_conf_push_group(key);
-
-    url = compat ? libbalsa_conf_get_string_with_default("URL", &def) :
-        libbalsa_urldecode(value);
-
-    if (!compat || !def) {
-        LibBalsaMailboxView *view;
-        gint tmp;
-        gchar *address;
-
-        view = libbalsa_mailbox_view_new();
-        /* In compatibility mode, mark as not in sync, to force
-         * the view to be saved in the config file. */
-        view->in_sync = !compat;
-        g_hash_table_insert(libbalsa_mailbox_view_table, g_strdup(url),
-                            view);
-
-        address =
-            libbalsa_conf_get_string_with_default("MailingListAddress",
-                                                  &def);
-        view->mailing_list_address =
-            def ? NULL : internet_address_list_parse_string(address);
-        g_free(address);
-
-        view->identity_name = libbalsa_conf_get_string("Identity");
-
-        tmp = libbalsa_conf_get_int_with_default("Threading", &def);
-        if (!def)
-            view->threading_type = tmp;
-
-        tmp = libbalsa_conf_get_int_with_default("GUIFilter", &def);
-        if (!def)
-            view->filter = tmp;
-
-        tmp = libbalsa_conf_get_int_with_default("SortType", &def);
-        if (!def)
-            view->sort_type = tmp;
-
-        tmp = libbalsa_conf_get_int_with_default("SortField", &def);
-        if (!def)
-            view->sort_field = tmp;
-
-        tmp = libbalsa_conf_get_int_with_default("Show", &def);
-        if (!def)
-            view->show = tmp;
-
-        tmp = libbalsa_conf_get_int_with_default("Subscribe", &def);
-        if (!def)
-            view->subscribe = tmp;
-
-        tmp = libbalsa_conf_get_bool_with_default("Exposed", &def);
-        if (!def)
-            view->exposed = tmp;
-
-        tmp = libbalsa_conf_get_bool_with_default("Open", &def);
-        if (!def)
-            view->open = tmp;
-#ifdef HAVE_GPGME
-        tmp = libbalsa_conf_get_int_with_default("CryptoMode", &def);
-        if (!def)
-            view->gpg_chk_mode = tmp;
-#endif
-
-        tmp = libbalsa_conf_get_int_with_default("Total", &def);
-        if (!def)
-            view->total = tmp;
-
-        tmp = libbalsa_conf_get_int_with_default("Unread", &def);
-        if (!def)
-            view->unread = tmp;
-
-        tmp = libbalsa_conf_get_int_with_default("ModTime", &def);
-        if (!def)
-            view->mtime = tmp;
-    }
-
-    libbalsa_conf_pop_group();
-    g_free(url);
-
-    return FALSE;
-}
-
-void
-config_views_load(void)
-{
-    /* Load old-style config sections in compatibility mode. */
-    libbalsa_conf_foreach_group(VIEW_SECTION_PREFIX,
-                                config_view_load,
-                                GINT_TO_POINTER(TRUE));
-    libbalsa_conf_foreach_group(VIEW_BY_URL_SECTION_PREFIX,
-                                config_view_load,
-                                GINT_TO_POINTER(FALSE));
-}
-
 /* Get viewByUrl prefix */
 static gchar *
 view_by_url_prefix(const gchar * url)
@@ -1852,11 +1751,83 @@ view_by_url_prefix(const gchar * url)
     return prefix;
 }
 
-/* config_views_save:
-   iterates over all mailbox views.
-*/
-static void
-save_view(const gchar * url, LibBalsaMailboxView * view)
+
+LibBalsaMailboxView *
+config_load_mailbox_view(const gchar * url)
+{
+    gchar *prefix;
+    LibBalsaMailboxView *view;
+
+    if (!url)
+        return NULL;
+
+    prefix = view_by_url_prefix(url);
+    if (!libbalsa_conf_has_group(prefix)) {
+        g_free(prefix);
+        return NULL;
+    }
+
+    libbalsa_conf_push_group(prefix);
+
+    view = libbalsa_mailbox_view_new();
+
+    if (libbalsa_conf_has_key("MailingListAddress")) {
+        gchar *address;
+
+        address = libbalsa_conf_get_string("MailingListAddress");
+        view->mailing_list_address =
+            internet_address_list_parse_string(address);
+        g_free(address);
+    }
+
+    view->identity_name = libbalsa_conf_get_string("Identity");
+
+    if (libbalsa_conf_has_key("Threading"))
+        view->threading_type = libbalsa_conf_get_int("Threading");
+
+    if (libbalsa_conf_has_key("GUIFilter"))
+        view->filter = libbalsa_conf_get_int("GUIFilter");
+
+    if (libbalsa_conf_has_key("SortType"))
+        view->sort_type = libbalsa_conf_get_int("SortType");
+
+    if (libbalsa_conf_has_key("SortField"))
+        view->sort_field = libbalsa_conf_get_int("SortField");
+
+    if (libbalsa_conf_has_key("Show"))
+        view->show = libbalsa_conf_get_int("Show");
+
+    if (libbalsa_conf_has_key("Subscribe"))
+        view->subscribe = libbalsa_conf_get_int("Subscribe");
+
+    if (libbalsa_conf_has_key("Exposed"))
+        view->exposed = libbalsa_conf_get_bool("Exposed");
+
+    if (libbalsa_conf_has_key("Open"))
+        view->open = libbalsa_conf_get_bool("Open");
+
+#ifdef HAVE_GPGME
+    if (libbalsa_conf_has_key("CryptoMode"))
+        view->gpg_chk_mode = libbalsa_conf_get_int("CryptoMode");
+#endif
+
+    if (libbalsa_conf_has_key("Total"))
+        view->total = libbalsa_conf_get_int("Total");
+
+    if (libbalsa_conf_has_key("Unread"))
+        view->unread = libbalsa_conf_get_int("Unread");
+
+    if (libbalsa_conf_has_key("ModTime"))
+        view->mtime = libbalsa_conf_get_int("ModTime");
+
+    libbalsa_conf_pop_group();
+    g_free(prefix);
+
+    return view;
+}
+
+void
+config_save_mailbox_view(const gchar * url, LibBalsaMailboxView * view)
 {
     gchar *prefix;
 
@@ -1896,7 +1867,8 @@ save_view(const gchar * url, LibBalsaMailboxView * view)
 	libbalsa_conf_set_int("Subscribe",   view->subscribe);
     if (view->exposed        != libbalsa_mailbox_get_exposed(NULL))
 	libbalsa_conf_set_bool("Exposed",    view->exposed);
-    if (view->open           != libbalsa_mailbox_get_open(NULL))
+    if (balsa_app.remember_open_mboxes &&
+        view->open           != libbalsa_mailbox_get_open(NULL))
 	libbalsa_conf_set_bool("Open",       view->open);
 #ifdef HAVE_GPGME
     if (view->gpg_chk_mode   != libbalsa_mailbox_get_crypto_mode(NULL))
@@ -1921,33 +1893,25 @@ save_view(const gchar * url, LibBalsaMailboxView * view)
     libbalsa_conf_pop_group();
 }
 
-void
-config_views_save(void)
-{
-    config_remove_groups(VIEW_SECTION_PREFIX);
-    /* save current */
-    g_hash_table_foreach(libbalsa_mailbox_view_table, (GHFunc) save_view,
-			 NULL);
-}
-
 void
 config_view_remove(const gchar * url)
 {
     gchar *prefix = view_by_url_prefix(url);
     libbalsa_conf_remove_group(prefix);
     g_free(prefix);
-    g_hash_table_remove(libbalsa_mailbox_view_table, url);
 }
 
 static void
-save_color(gchar * key, GdkColor * color)
+save_color(const gchar * key, GdkRGBA * rgba)
 {
+    gchar *full_key;
     gchar *str;
 
-    str = g_strdup_printf("#%04x%04x%04x", color->red, color->green,
-                          color->blue);
-    libbalsa_conf_set_string(key, str);
+    full_key = g_strconcat(key, "RGBA", NULL);
+    str = gdk_rgba_to_string(rgba);
+    libbalsa_conf_set_string(full_key, str);
     g_free(str);
+    g_free(full_key);
 }
 
 static gboolean
@@ -2015,7 +1979,7 @@ config_filters_save(void)
 	libbalsa_filter_save_config(fil);
 	libbalsa_conf_pop_group();
     }
-    libbalsa_conf_sync();
+    libbalsa_conf_queue_sync();
     /* This loop takes care of cleaning up old filter sections */
     while (TRUE) {
 	i=snprintf(tmp,tmp_len,"%d",nb++);
@@ -2026,7 +1990,7 @@ config_filters_save(void)
 	}
 	else break;
     }
-    libbalsa_conf_sync();
+    libbalsa_conf_queue_sync();
     g_free(buffer);
 }
 
@@ -2057,19 +2021,25 @@ config_mailbox_filters_save(LibBalsaMailbox * mbox)
     }
     libbalsa_mailbox_filters_save_config(mbox);
     libbalsa_conf_pop_group();
-    libbalsa_conf_sync();
+    libbalsa_conf_queue_sync();
 }
 
 static void
-load_color(gchar * key, GdkColor * color)
+load_color(const gchar * key, const gchar * def_val, GdkRGBA * rgba)
 {
+    gchar *full_key;
     gchar *str;
+    gboolean def_used;
 
-    str = libbalsa_conf_get_string(key);
-    if (g_ascii_strncasecmp(str, "rgb:", 4)
-        || sscanf(str + 4, "%4hx/%4hx/%4hx", &color->red, &color->green,
-                  &color->blue) != 3)
-        gdk_color_parse(str, color);
+    full_key = g_strdup_printf("%sRGBA", key);
+    str = libbalsa_conf_get_string(full_key);
+    if (!str) {
+        g_free(full_key);
+        full_key = g_strdup_printf("%s=%s", key, def_val);
+        str = libbalsa_conf_get_string_with_default(full_key, &def_used);
+    }
+    g_free(full_key);
+    gdk_rgba_parse(rgba, str);
     g_free(str);
 }
 
@@ -2087,8 +2057,9 @@ load_mru(GList **mru, const gchar * group)
         gchar *val;
 	snprintf(tmpkey, sizeof tmpkey - 1, "MRU%d", i + 1);
         if( (val = libbalsa_conf_get_string(tmpkey)) != NULL )
-            (*mru)=g_list_append((*mru), val);
+            (*mru)=g_list_prepend((*mru), val);
     }
+    *mru = g_list_reverse(*mru);
     libbalsa_conf_pop_group();
 }
 
@@ -2111,88 +2082,63 @@ save_mru(GList * mru, const gchar * group)
     libbalsa_conf_pop_group();
 }
 
-/* check_for_old_sigs:
-   function for old style signature conversion (executable sigs prefixed
-   with '|') to new style (filename and a checkbox).
-   this function just strips the leading '|'.
-*/
-static void 
-check_for_old_sigs(GList * id_list_tmp)
-{
-    /* strip pipes and spaces,set executable flag if warranted */
-    /* FIXME remove after a few stable releases.*/
-    
-    LibBalsaIdentity* id_tmp = NULL;
-    
-    for (id_list_tmp = balsa_app.identities; id_list_tmp; 
-         id_list_tmp = id_list_tmp->next) {
-       
-        id_tmp = LIBBALSA_IDENTITY(id_list_tmp->data);
-        if(!id_tmp->signature_path) continue;
-
-        id_tmp->signature_path = g_strstrip(id_tmp->signature_path);
-        if(*id_tmp->signature_path == '|'){
-            printf("Found old style signature for identity: %s\n"\
-                   "Converting: %s --> ", id_tmp->identity_name, 
-                   id_tmp->signature_path);
-            id_tmp->signature_path = g_strchug(id_tmp->signature_path+1);
-            printf("%s \n", id_tmp->signature_path);
-            
-            /* set new-style executable var*/
-            id_tmp->sig_executable=TRUE;
-            printf("Setting converted signature as executable.\n");
-        }
-    }
-}
-
-#if HAVE_GNOME
 void
 config_defclient_save(void)
 {
-    static struct {
-        const char *key, *val;
-    } gconf_string[] = {
-        {"/desktop/gnome/url-handlers/mailto/command",     "balsa -m \"%s\""},
-        {"/desktop/gnome/url-handlers/mailto/description", "Email" }};
-    static struct {
-        const char *key; gboolean val;
-    } gconf_bool[] = {
-        {"/desktop/gnome/url-handlers/mailto/need-terminal", FALSE},
-        {"/desktop/gnome/url-handlers/mailto/enabled",       TRUE}};
+    GDesktopAppInfo *info;
+    GError *err;
 
-    if (balsa_app.default_client) {
-        GError *err = NULL;
-        GConfClient *gc;
-        unsigned i;
-        gc = gconf_client_get_default(); /* FIXME: error handling */
-        if (gc == NULL) {
-            balsa_information(LIBBALSA_INFORMATION_WARNING,
-                              _("Error opening GConf database\n"));
-            return;
-        }
-        for(i=0; i<ELEMENTS(gconf_string); i++) {
-            gconf_client_set_string(gc, gconf_string[i].key, 
-                                    gconf_string[i].val, &err);
-            if (err) {
-                balsa_information(LIBBALSA_INFORMATION_WARNING,
-                                  _("Error setting GConf field: %s\n"),
-                                  err->message);
-                g_error_free(err);
-                return;
-            }
-        }
-        for(i=0; i<ELEMENTS(gconf_bool); i++) {
-            gconf_client_set_bool(gc, gconf_bool[i].key,
-                                  gconf_bool[i].val, &err);
-            if (err) {
-                balsa_information(LIBBALSA_INFORMATION_WARNING,
-                                  _("Error setting GConf field: %s\n"),
-                                  err->message);
-                g_error_free(err);
-                return;
-            }
-            g_object_unref(gc);
-        }
+    if (!balsa_app.default_client)
+        return;
+
+    info = g_desktop_app_info_new("balsa.desktop");
+    if (!info) {
+        g_warning("Failed to create default application for Balsa "
+                  "for \"mailto\"");
+        return;
     }
+
+    err = NULL;
+    if (!g_app_info_set_as_default_for_type
+        (G_APP_INFO(info), "x-scheme-handler/mailto", &err)) {
+        g_warning("Failed to set Balsa as the default application "
+                  "for \"mailto\": %s", err->message);
+        g_error_free(err);
+    }
+    g_object_unref(info);
+}
+
+static gboolean
+config_mailbox_had_property(const gchar * url, const gchar * key)
+{
+    gchar *prefix;
+    gboolean retval = FALSE;
+
+    prefix = view_by_url_prefix(url);
+    if (!libbalsa_conf_has_group(prefix)) {
+        g_free(prefix);
+        return retval;
+    }
+
+    libbalsa_conf_push_group(prefix);
+
+    if (libbalsa_conf_has_key(key))
+        retval = libbalsa_conf_get_bool(key);
+
+    libbalsa_conf_pop_group();
+    g_free(prefix);
+
+    return retval;
+}
+
+gboolean
+config_mailbox_was_open(const gchar * url)
+{
+    return config_mailbox_had_property(url, "Open");
+}
+
+gboolean
+config_mailbox_was_exposed(const gchar * url)
+{
+    return config_mailbox_had_property(url, "Exposed");
 }
-#endif /* HAVE_GNOME */
