@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2002 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -93,7 +93,6 @@ extern "C" {
         gulong delete_range_sig_id;
 #endif                          /* HAVE_GTKSOURCEVIEW */
         gulong insert_text_sig_id;
-        guint wrap_timeout_id;
         guint autosave_timeout_id;
         SendmsgState state;
         gulong identities_changed_id;
@@ -102,13 +101,9 @@ extern "C" {
 	gboolean req_dispnotify; /* send a MDN */ 
 	gboolean quit_on_close; /* quit balsa after the compose window */
 	                        /* is closed.                          */
-        /* style for changing the color of address labels when the
-         * address isn't valid: */
-        GtkStyle *bad_address_style;  
 #ifdef HAVE_GPGME
 	guint gpg_mode;
 #endif
-        GtkWidget *header_table;
 
 #if !HAVE_GTKSOURCEVIEW
         GtkTextBuffer *buffer2;       /* Undo buffer. */
