@@ -1,6 +1,6 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2003 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -276,45 +276,40 @@ store_address_from_entries(GtkWindow *window, StoreAddressInfo * info,
 static GtkWidget *
 store_address_book_frame(StoreAddressInfo * info)
 {
-    GList *ab_list;
-    GtkWidget *frame = gtk_frame_new(_("Choose Address Book"));
+    GtkWidget *frame;
     GtkWidget *combo_box;
-    LibBalsaAddressBook *address_book;
-    guint default_ab_offset = 0, off;
 
-#if GTK_CHECK_VERSION(2, 24, 0)
     combo_box = gtk_combo_box_text_new();
-#else                           /* GTK_CHECK_VERSION(2, 24, 0) */
-    combo_box = gtk_combo_box_new_text();
-#endif                          /* GTK_CHECK_VERSION(2, 24, 0) */
     g_signal_connect(combo_box, "changed",
                      G_CALLBACK(store_address_book_menu_cb), info);
     if (balsa_app.address_book_list) {
+        guint default_ab_offset = 0, off;
+        GList *ab_list;
+
 	info->current_address_book = balsa_app.default_address_book;
 
 	/* NOTE: we have to store the default address book index and
            call set_active() after all books are added to the list or
            gtk-2.10.4 will lose the setting. */
 	for(off=0, ab_list = balsa_app.address_book_list;
-            ab_list; 
-            off++, ab_list = g_list_next(ab_list)) {
+            ab_list;
+            off++, ab_list = ab_list->next) {
+            LibBalsaAddressBook *address_book;
+
 	    address_book = LIBBALSA_ADDRESS_BOOK(ab_list->data);
 	    if (info->current_address_book == NULL)
 		info->current_address_book = address_book;
 
-#if GTK_CHECK_VERSION(2, 24, 0)
-            gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo_box),
+	    gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo_box),
                                            address_book->name);
-#else                           /* GTK_CHECK_VERSION(2, 24, 0) */
-	    gtk_combo_box_append_text(GTK_COMBO_BOX(combo_box),
-                                      address_book->name);
-#endif                          /* GTK_CHECK_VERSION(2, 24, 0) */
 	    if (address_book == balsa_app.default_address_book)
                 default_ab_offset = off;
 	}
         gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box),
                                  default_ab_offset);
     }
+
+    frame = gtk_frame_new(_("Choose Address Book"));
     gtk_container_add(GTK_CONTAINER(frame), combo_box);
 
     return frame;
@@ -381,7 +376,28 @@ store_address_add_address(StoreAddressInfo * info,
     address = libbalsa_address_new();
     address->full_name =
         g_strdup(ia->name ? ia->name : group ? group->name : NULL);
-    address->address_list = g_list_prepend(NULL, g_strdup(INTERNET_ADDRESS_MAILBOX (ia)->addr));
+    if (INTERNET_ADDRESS_IS_GROUP(ia)) {
+        InternetAddressList *members;
+        int j;
+
+        address->address_list = NULL;
+        members = INTERNET_ADDRESS_GROUP(ia)->members;
+
+        for (j = 0; j < internet_address_list_length(members); j++) {
+            InternetAddress *member_address =
+                internet_address_list_get_address(members, j);
+            if (INTERNET_ADDRESS_IS_MAILBOX(member_address))
+                address->address_list =
+                    g_list_prepend(address->address_list,
+                                   g_strdup(INTERNET_ADDRESS_MAILBOX
+                                            (member_address)->addr));
+        }
+        address->address_list = g_list_reverse(address->address_list);
+    } else {
+        address->address_list =
+            g_list_prepend(NULL,
+                           g_strdup(INTERNET_ADDRESS_MAILBOX(ia)->addr));
+    }
     ew = libbalsa_address_get_edit_widget(address, entries, NULL, NULL);
     g_object_unref(address);
 
@@ -422,27 +438,31 @@ store_address_add_lbaddress(StoreAddressInfo * info,
  * take a list of addresses and pass them one at a time to
  * store_address_add_address */
 static void
-store_address_add_list(StoreAddressInfo * info,
-                       const gchar * label,
+store_address_add_list(StoreAddressInfo    * info,
+                       const gchar         * label,
                        InternetAddressList * list)
 {
     int i, j;
-    
+
     if (!list)
         return;
 
-    for (i = 0; i < internet_address_list_length (list); i++) {
-        InternetAddress *ia = internet_address_list_get_address (list, i);
-	
-        if (INTERNET_ADDRESS_IS_MAILBOX (ia))
+    for (i = 0; i < internet_address_list_length(list); i++) {
+        InternetAddress *ia = internet_address_list_get_address(list, i);
+
+        if (INTERNET_ADDRESS_IS_MAILBOX(ia)) {
             store_address_add_address(info, label, ia, NULL);
-        else {
-            InternetAddressList *members = INTERNET_ADDRESS_GROUP (ia)->members;
+        } else if (info->current_address_book->dist_list_mode) {
+            store_address_add_address(info, label, ia, ia);
+        } else {
+            InternetAddressList *members =
+                INTERNET_ADDRESS_GROUP(ia)->members;
 
-            for (j = 0; j < internet_address_list_length (members); j++) {
-                InternetAddress *member_address = internet_address_list_get_address (members, j);
+            for (j = 0; j < internet_address_list_length(members); j++) {
+                InternetAddress *member_address =
+                    internet_address_list_get_address(members, j);
 
-                if (INTERNET_ADDRESS_IS_MAILBOX (member_address))
+                if (INTERNET_ADDRESS_IS_MAILBOX(member_address))
                     store_address_add_address(info, label, member_address,
                                               ia);
             }
