@@ -1,21 +1,21 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1997-2001 Stuart Parmenter and others,
+ * Copyright (C) 1997-2013 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option) 
+ * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- *  
+ *
  * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *  
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -25,10 +25,6 @@
 #include "toolbar-factory.h"
 
 #include <string.h>
-#if HAVE_GNOME
-#include <gconf/gconf-client.h>
-#endif
-
 #include <glib/gi18n.h>
 
 #include "balsa-app.h"
@@ -56,6 +52,9 @@ struct BalsaToolbarModel_ {
     GSList          *current;
     BalsaToolbarType type;
     GtkToolbarStyle  style;
+#if HAVE_GNOME
+    GSettings       *settings;
+#endif /* HAVE_GNOME */
 };
 
 enum {
@@ -63,7 +62,7 @@ enum {
     LAST_SIGNAL
 };
 
-static GtkObjectClass* parent_class;
+static GObjectClass* parent_class;
 static guint model_signals[LAST_SIGNAL] = { 0 };
 
 static void
@@ -71,6 +70,9 @@ balsa_toolbar_model_finalize(GObject * object)
 {
     BalsaToolbarModel *model = BALSA_TOOLBAR_MODEL(object);
     g_hash_table_destroy(model->legal);
+#if HAVE_GNOME
+    g_object_unref(model->settings);
+#endif /* HAVE_GNOME */
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
@@ -122,7 +124,7 @@ balsa_toolbar_model_get_type()
                                    "BalsaToolbarModel",
                                    &balsa_toolbar_model_info, 0);
     }
-    
+
     return balsa_toolbar_model_type;
 }
 
@@ -201,7 +203,7 @@ void
 balsa_toolbar_remove_all(GtkWidget * widget)
 {
     GList *child, *children;
-    
+
     children = gtk_container_get_children(GTK_CONTAINER(widget));
     for (child = children; child; child = child->next)
         gtk_widget_destroy(child->data);
@@ -268,13 +270,17 @@ tm_save_model(BalsaToolbarModel * model)
 }
 
 #if HAVE_GNOME
-/* GConfClientNotifyFunc
+/* GSettings change_cb
  */
 static void
-tm_gconf_notify(GConfClient * client, guint cnxn_id, GConfEntry * entry,
-                BalsaToolbarModel * model)
+tm_gsettings_change_cb(GSettings   * settings,
+                       const gchar * key,
+                       gpointer      user_data)
 {
-    if (model->style == (GtkToolbarStyle) (-1))
+    BalsaToolbarModel *model = user_data;
+
+    if (!strcmp(key, "toolbar-style") &&
+        model->style == (GtkToolbarStyle) (-1))
         balsa_toolbar_model_changed(model);
 }
 #endif /* HAVE_GNOME */
@@ -286,23 +292,15 @@ balsa_toolbar_model_new(BalsaToolbarType type, GSList * standard)
 {
     BalsaToolbarModel *model =
         g_object_new(BALSA_TYPE_TOOLBAR_MODEL, NULL);
-#if HAVE_GNOME
-    GConfClient *conf;
-    guint notify_id;
-#endif
 
     model->type = type;
     model->standard = standard;
     tm_load_model(model);
 
 #if HAVE_GNOME
-    conf = gconf_client_get_default();
-    /* We never destroy a model, so we do nothing with the notify-id: */
-    notify_id =
-        gconf_client_notify_add(conf,
-                                "/desktop/gnome/interface/toolbar_style",
-                                (GConfClientNotifyFunc) tm_gconf_notify,
-                                model, NULL, NULL);
+    model->settings = g_settings_new("org.gnome.desktop.interface");
+    g_signal_connect(model->settings, "changed",
+                     G_CALLBACK(tm_gsettings_change_cb), model);
 #endif /* HAVE_GNOME */
 
     return model;
@@ -469,15 +467,14 @@ static void
 tm_populate(BalsaToolbarModel * model, GtkUIManager * ui_manager,
             GArray * merge_ids)
 {
-    gboolean style_is_both_horiz;
+    gboolean style_is_both;
     gboolean make_two_line;
     GSList *list;
 
-    style_is_both_horiz = (model->style == GTK_TOOLBAR_BOTH_HORIZ
-                           || (model->style == (GtkToolbarStyle) -1
-                               && tm_default_style() ==
-                               GTK_TOOLBAR_BOTH_HORIZ));
-    make_two_line = !style_is_both_horiz && tm_has_second_line(model);
+    style_is_both = (model->style == GTK_TOOLBAR_BOTH
+                     || (model->style == (GtkToolbarStyle) -1
+                         && tm_default_style() == GTK_TOOLBAR_BOTH));
+    make_two_line = style_is_both && tm_has_second_line(model);
 
     for (list = balsa_toolbar_model_get_current(model); list;
          list = list->next) {
@@ -536,14 +533,12 @@ tm_default_style(void)
 {
     GtkToolbarStyle default_style = GTK_TOOLBAR_BOTH;
 #if HAVE_GNOME
-    GConfClient *conf;
+    GSettings *settings;
     gchar *str;
 
     /* Get global setting */
-    conf = gconf_client_get_default();
-    str  = gconf_client_get_string(conf,
-                                   "/desktop/gnome/interface/toolbar_style",
-                                   NULL);
+    settings = g_settings_new("org.gnome.desktop.interface");
+    str  = g_settings_get_string(settings, "toolbar-style");
     if (str) {
         guint i;
 
@@ -554,6 +549,7 @@ tm_default_style(void)
             }
         g_free(str);
     }
+    g_object_unref(settings);
 #endif /* HAVE_GNOME */
 
     return default_style;
@@ -664,24 +660,17 @@ tm_popup_position_func(GtkMenu * menu, gint * x, gint * y,
     GtkRequisition req;
     gint monitor_num;
     GdkRectangle monitor;
-#if GTK_CHECK_VERSION(2, 18, 0)
     GtkAllocation allocation;
-#endif                          /* GTK_CHECK_VERSION(2, 18, 0) */
 
     g_return_if_fail(gtk_widget_get_window(toolbar));
 
     gdk_window_get_origin(gtk_widget_get_window(toolbar), x, y);
 
-    gtk_widget_size_request(GTK_WIDGET(menu), &req);
+    gtk_widget_get_preferred_size(GTK_WIDGET(menu), NULL, &req);
 
-#if GTK_CHECK_VERSION(2, 18, 0)
     gtk_widget_get_allocation(toolbar, &allocation);
     *x += (allocation.width - req.width) / 2;
     *y += allocation.height;
-#else                           /* GTK_CHECK_VERSION(2, 18, 0) */
-    *x += (toolbar->allocation.width - req.width) / 2;
-    *y += toolbar->allocation.height;
-#endif                          /* GTK_CHECK_VERSION(2, 18, 0) */
 
     monitor_num = gdk_screen_get_monitor_at_point(screen, *x, *y);
     gtk_menu_set_monitor(menu, monitor_num);
@@ -769,11 +758,7 @@ tm_do_popup_menu(GtkWidget * toolbar, GdkEventButton * event,
         }
     }
 
-#if GTK_CHECK_VERSION(2, 18, 0)
     if (gtk_widget_is_sensitive(toolbar)) {
-#else                           /* GTK_CHECK_VERSION(2, 18, 0) */
-    if (GTK_WIDGET_IS_SENSITIVE(toolbar)) {
-#endif                          /* GTK_CHECK_VERSION(2, 18, 0) */
         /* This is a real toolbar, not the template from the
          * toolbar-prefs dialog. */
         GtkWidget *item;
